using DocumentFormat.OpenXml.Bibliography;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.Text;
using System.Text.Json.Serialization;
using TEAMModelOS.SDK.Context.Attributes.Azure;
using TEAMModelOS.SDK.DI;

namespace TEAMModelFunction
{
    /// <summary>
    /// 考试基础信息
    /// </summary>
    //[CosmosDB(Database = "TEAMModelOS", Name = "School")]
    public class ExamInfo : ID
    {
        [PartitionKey]
        [Required(ErrorMessage = "{0} 必须填写")]
        public string code { get; set; }
    
        public string pk { get; set; }
        public int? ttl { get; set; } 
        public ExamInfo() {
            period = new PeriodSimple();
            grades = new List<Grade>();
            subjects = new List<Subject>();
            papers = new List<PaperSimple>();
            targetClassIds = new List<string>();
        }
        [JsonProperty(PropertyName = "id")]
        public string id { get; set; }

        public string name { get; set; }
        public string school { get; set; }
        public string creatorId { get; set; }
        public int stuCount { get; set; }
/*        //实际考试人数
        public int realCount { get; set; }
        //平均分
        public double piont { get; set; }*/
        //指标
        //public double index { get; set; }
        public long createTime { get; set; }
        //public string examCode { get; set; }
        
        /// <summary>
        /// 施测起始时间
        /// </summary>
        public long startTime { get; set; }
        /// <summary>
        /// 施测结束时间
        /// </summary>
        public long endTime { get; set; }
        /// <summary>
        /// 施测对象
        /// </summary>
       // public List<string> target { get; set; }
        /// <summary>
        /// 发布
        /// </summary>
        public string publish { get; set; }

        public int status { get; set; }
        public int year { get; set; }
        public string range { get; set; }
        public string source { get; set; }
        public List<string> targetClassIds { get; set; }
        public List<PaperSimple> papers { get; set; }
        ///考试类型 段考 stage  联考 union 平常考 normal 其他 other
        /// </summary>
        public string type { get; set; }
        public PeriodSimple period { get; set; }
        public List<Grade> grades { get; set; }
        public List<Subject> subjects { get; set; }
        public long sequenceNumber { get; set; }
        //public Condition conditions { get; set; }
        public List<string> blobUrl { get; set; }
        public string progress { get; set; }
        public string scope { get; set; }
        public Custom examType { get; set; }
        /// <summary>
        /// 所有试卷阅卷状态
        /// </summary>
        //public List<int> marks { get; set; }
    }
    public class Custom {
        public string id { get; set; }
        public string name { get; set; }
    }
    public class PaperSimple { 
        public string id { get; set; }
        public string code { get; set; }
        public string name { get; set; }
        public string blob { get; set; }
        public string scope { get; set; }
        public int multipleRule { get; set; }
        //该试卷配分情况
        public List<double> point { get; set; }
        public List<List<string>> answers { get; set; }
    }
    public class PeriodSimple
    {
        public string id { get; set; }
        public string name { get; set; }
    }
    public class Grade
    {
        public string id { get; set; }
        public string name { get; set; }
    }
    public class Subject
    {
        public string id { get; set; }
        public string name { get; set; }


    }
}
