﻿using Google.Protobuf;
using Microsoft.Azure.Cosmos.Linq;
using ProtoBuf;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using TEAMModelOS.SDK.Helper.Common.CollectionHelper;

namespace TEAMModelGrpc.Models
{
    /// <summary>
    /// 请求参数Dict
    /// </summary>
    [ProtoContract]
    public class Dict
    {
        /// <summary>
        /// 数字Dict
        /// </summary>
        [ProtoMember(1)]
        public Dictionary<string, double> NMap { get; set; }
        /// <summary>
        /// 字符串Dict
        /// </summary>
        [ProtoMember(2)]
        public Dictionary<string, string> SMap { get; set; }
        /// <summary>
        /// 布尔Dict
        /// </summary>
        [ProtoMember(3)] 
        public Dictionary<string, bool> BMap { get; set; }
        /// <summary>
        /// 字符串ListMap
        /// </summary>
        [ProtoMember(4)]
        public List<LSMap> LSMap { get; set; }
        /// <summary>
        /// 数字ListMap
        /// </summary>
        [ProtoMember(5)]
        public List<LNMap> LNMap { get; set; }
        /// <summary>
        /// Byte ListMap
        /// </summary>
        [ProtoMember(6)]
        public List<LBMap> LBMap { get; set; }

        


    }
    /// <summary>
    /// 字符串ListMap
    /// </summary>
    [ProtoContract]
    public class LSMap
    {
        /// <summary>
        /// 数字Dict
        /// </summary>
        [ProtoMember(1)]
        public string Key { get; set; }
        /// <summary>
        /// 字符串数组
        /// </summary>
        [ProtoMember(2)]
        public List<string> Value {get;set;}
    }
    /// <summary>
    /// 数字ListMap
    /// </summary>
    [ProtoContract]
    public class LNMap
    {
        /// <summary>
        /// Key
        /// </summary>
        [ProtoMember(1)]
        public string Key { get; set; }
        /// <summary>
        /// 数字数组
        /// </summary>
        [ProtoMember(2)]
        public List<double> Value { get; set; }
    }
     
    /// <summary>
    /// Byte[] Map
    /// </summary>
    [ProtoContract]
    public class LBMap
    {
        /// <summary>
        /// Key
        /// </summary>
        [ProtoMember(1)]
        public string Key { get; set; }
        /// <summary>
        /// byte数组
        /// </summary>
        [ProtoMember(2)]
        public byte[] Value { get; set; }
    }
    /// <summary>
    ///  Dict扩展
    /// </summary>
    public static class DictExt {
        /// <summary>
        ///  Dict扩展方法
        /// </summary>
        /// <param name="dict"></param>
        /// <returns></returns>
        public static Dictionary<string, object> ToDict(this Dict dict)
        {

            Dictionary<string, object> keyValues = new Dictionary<string, object>();
            if (dict.SMap != null && dict.SMap.Keys.Count > 0)
            {
                foreach (string key in dict.SMap.Keys)
                {
                    keyValues[key] = dict.SMap[key];
                }
            }
            if (dict.NMap != null && dict.NMap.Keys.Count > 0)
            {
                foreach (string key in dict.NMap.Keys)
                {
                    keyValues[key] = dict.NMap[key];
                }
            }
            if (dict.BMap != null && dict.BMap.Keys.Count > 0)
            {
                foreach (string key in dict.BMap.Keys)
                {
                    keyValues[key] = dict.BMap[key];
                }
            }
            if (dict.LSMap.IsNotEmpty())
            {
                foreach (LSMap map in dict.LSMap)
                {
                    if (!string.IsNullOrWhiteSpace(map.Key)&& map.Value.IsNotEmpty()) {
                        keyValues[map.Key] = map.Value;
                    }
                }
            }

            if (dict.LNMap.IsNotEmpty())
            {
                foreach (LNMap map in dict.LNMap)
                {
                    if (!string.IsNullOrWhiteSpace(map.Key) && map.Value.IsNotEmpty())
                    {
                        keyValues[map.Key] = map.Value;
                    }
                }
            }
            if (dict.LBMap.IsNotEmpty())
            {
                foreach (LBMap map in dict.LBMap)
                {
                    if (!string.IsNullOrWhiteSpace(map.Key) && map.Value.IsNotEmpty())
                    {
                        keyValues[map.Key] = map.Value;
                    }
                }
            }
            return keyValues;
        }
    }
}
