using Grpc.Core;
using Grpc.Extension.Abstract;
using Microsoft.AspNetCore.Authorization;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using TEAMModelGrpc.Models;
using TEAMModelOS.SDK.Helper.Common.CollectionHelper;
using TEAMModelOS.SDK.DI;
using TEAMModelOS.Models.StudentInfo;

namespace TEAMModelGrpc.Services
{
    public class ClassStudentService : IGrpcService
    {
        private AzureCosmosFactory _azureCosmos;
        public ClassStudentService(AzureCosmosFactory azureCosmos)
        {
            _azureCosmos = azureCosmos;
        }

        /// <summary>
        /// 保存 或 修改 教室学生关联
        /// </summary>
        /// <param name="requestStream"></param>
        /// <param name="responseStream"></param>
        /// <param name="context"></param>
        /// <returns></returns>
        [Authorize]
        public async Task SaveOrUpdateClassStudent(IAsyncStreamReader<ClassStudent> requestStream, IServerStreamWriter<ClassStudent> responseStream, ServerCallContext context)
        {
            List<ClassStudent> classrooms = new List<ClassStudent>();
            await foreach (ClassStudent students in requestStream.ReadAllAsync())
            {
                classrooms.Add(await _azureCosmos.SaveOrUpdate(students));
            }
            classrooms.ForEach(x => {
                responseStream.WriteAsync(x);
            });
        }



        



        /// <summary>
        /// 查询学生在哪些教室
        /// </summary>
        /// <param name="request"></param>
        /// <param name="responseStream"></param>
        /// <param name="context"></param>
        /// <returns></returns>
        [Authorize]
        public async Task FindStudents(StringDto request, IServerStreamWriter<ClassStudent> responseStream, ServerCallContext context) 
        {
            List<ClassStudent> ClassStudents = await _azureCosmos.FindSQL<ClassStudent>("select c.id,c.code from c join A0  in c.studentId where 1=1 and c.pk='ClassStudent' and A0 =  \'" + request + "\'");
            if (ClassStudents.IsNotEmpty()) {
                ClassStudents.ForEach(  x => { 
                responseStream.WriteAsync(x);
                });
            }
        }
    }
}
