import ServerSideLogin from '@/view/serverside/login.vue'
import Home from '@/view/Home.vue'
import FrontEndMain from '@/components/public/frontEndMain/Index.vue'
// import HTTP404 from '@/view/404'
// import { resolve } from 'url'

let identity = localStorage.getItem('identity')

export const routes = [
	{
		path: '*',
		redirect: '/404'
	},
	{
		name: '404',
		path: '/404',
		component: resolve => require(['@/view/404.vue'], resolve)
	},
	{
		name: 'index',
		path: '',
		redirect: identity == 'student' ? '/studentWeb' : '/home'
	},
	{
		path: '/login',
		component: FrontEndMain,
		children: [{
			name: 'login',
			path: '/',
			meta: {
				middleware: ['login?']
			},
			component: resolve => require(['@/view/login/Index.vue'], resolve)
		}]
	},
	{
		path: '/regist',
		component: FrontEndMain,
		children: [{
			name: 'regist',
			path: '/',
			meta: {
				middleware: ['login?']
			},
			component: resolve => require(['@/view/regist/Index.vue'], resolve)
		}]
	},
	{
		path: '/forgotpw',
		component: FrontEndMain,
		children: [{
			name: 'forgotpw',
			path: '/',
			meta: {
				middleware: ['login?']
			},
			component: resolve => require(['@/view/forgotPw/Index.vue'], resolve)
		}]
	},
	{
		path: '/schoolList',
		component: Home,
		children: [{
			name: 'schoolList',
			path: '/',
			meta: {
				middleware: ['login']
			},
			component: resolve => require(['@/view/schoolList/Index.vue'], resolve),
		}]
	},
	{
		name: 'ServerSideLogin',
		path: '/serverside/login',
		component: ServerSideLogin
	},
	{
		name: 'smartschooldashboard',
		path: '/smartschooldashboard',
		component: resolve => require(['@/view/smartschooldashboard/Index.vue'], resolve)
	},
	{
		name: 'smartclassdashboard',
		path: '/smartclassdashboard',
		component: resolve => require(['@/view/smartclassdashboard/Index.vue'], resolve)
	},
	{
		name: 'embedschooldashboard',
		path: '/embedclassdashboard/:schoolshortcode',
		component: resolve => require(['@/view/smartclassdashboard/Index.vue'], resolve)
	}, // IES 呼叫專用

	// 教师端
	{
		name: 'home',
		path: '/home',
		redirect: '/home/HomePage',
		component: resolve => require(['@/view/Home'], resolve),
		meta: {
			middleware: ['login', 'role:admin|teacher'], // 强制登录（会自动跳转到登录页），角色为管理员或教师
		},
		children: [{
				name: 'totalIndex',
				path: '/totalIndex',
				component: resolve => require(['@/view/student-analysis/total-analysis/EvaluationList/TotalIndex.vue'], resolve),
			},
			{
				name: 'total',
				path: 'total',
				component: resolve => require(['@/view/student-analysis/total-analysis/index.vue'], resolve),
				children: [{
						path: '/total',
						component: resolve => require([
							'@/view/student-analysis/total-analysis/AchievementAnalysis/AchievementAnalysis.vue'
						], resolve)
					},
					{
						path: '/total/evaluationList',
						component: resolve => require(['@/view/student-analysis/total-analysis/EvaluationList/EvaluationList.vue'],
							resolve),
					},
					{
						path: '/total/achievement',
						component: resolve => require([
							'@/view/student-analysis/total-analysis/AchievementAnalysis/AchievementAnalysis.vue'
						], resolve),
					},
					{
						path: '/total/achievement/entryTables',
						component: resolve => require(['@/view/student-analysis/total-analysis/AchievementAnalysis/EntryTables.vue'],
							resolve),
					},
					{
						// name: 'earlyWarning',
						path: '/total/achievement/earlyWarning',
						component: resolve => require(['@/view/student-analysis/total-analysis/AchievementAnalysis/EarlyWarning.vue'],
							resolve)
					},
					{
						path: '/total/scatter',
						component: resolve => require(['@/view/student-analysis/total-analysis/ScatterAnalysis/ScatterAnalysis.vue'],
							resolve),
					},
					{
						path: '/total/knowledge',
						component: resolve => require([
							'@/view/student-analysis/total-analysis/KnowledgeAnalysis/KnowledgeAnalysis.vue'
						], resolve)
					},
					{
						path: '/total/knowledge/details',
						component: resolve => require(['@/view/student-analysis/total-analysis/KnowledgeAnalysis/ScoreDetails.vue'],
							resolve)
					},
					{
						path: '/total/cognitionLevel',
						component: resolve => require(['@/view/student-analysis/total-analysis/LevelAnalysis/LevelAnalysis.vue'],
							resolve)
					},
					{
						path: '/total/cognitionLevel/details',
						component: resolve => require(['@/view/student-analysis/total-analysis/LevelAnalysis/ScoreDetails.vue'],
							resolve)
					},
					{
						path: '/total/test',
						component: resolve => require(['@/view/student-analysis/total-analysis/TestAnalysis/TestAnalysis.vue'],
							resolve)
					},
					{
						path: '/total/questionList',
						component: resolve => require(['@/view/student-analysis/total-analysis/TestAnalysis/QuestionList.vue'],
							resolve)
					}
				]
			},
			//校园基础数据管理
			{
				name: 'system',
				path: 'system',
				meta: {
					middleware: ['login', 'ability:admin,schoolSetting-read|schoolSetting-upd']
				},
				component: resolve => require(['@/view/schoolmgmt/SystemSetting/SystemSetting.vue'], resolve)
			},
			//班级教室管理
			{
				name: 'classroom',
				path: 'classroom',
				meta: {
					middleware: ['login', 'ability:admin,classroom-read|classroom-upd']
				},
				component: resolve => require(['@/view/schoolmgmt/ClassroomSetting/ClassroomSetting.vue'], resolve)
			},
			//学生账号管理
			{
				name: 'studentAccount',
				path: 'studentAccount',
				component: resolve => require(['@/view/student-account/Index.vue'], resolve),
				meta: {
					middleware: ['login', 'ability:admin,student-read|student-upd']
				},

			},
			//教师账号
			{
				path: 'teachermgmt',
				name: 'teachermgmt',
				meta: {
					middleware: ['login', 'ability:admin,teacher-read|teacher-upd']
				},
				component: () => import('@/view/teachermgmt/Index.vue')
			},
			{
				path: 'evaluation',
				name: 'evaluation',
				redirect: '/home/evaluation/testPaperList',
				component: resolve => require(['@/view/evaluation/index/index.vue'], resolve),
				children: [{
						path: 'createExercises',
						name: 'createExercises',
						component: resolve => require(['@/view/evaluation/index/CreateExercises.vue'], resolve)
					},
					{
						path: 'testPaper',
						name: 'testPaper',
						component: resolve => require(['@/view/evaluation/index/TestPaper.vue'], resolve)
					},
					{
						path: 'testPaperList',
						name: 'testPaperList',
						component: resolve => require(['@/view/evaluation/bank/index.vue'], resolve)
					},
					{
						path: 'schoolBank',
						name: 'schoolBank',
						component: resolve => require(['@/view/evaluation/bank/index.vue'], resolve)
					},
					{
						path: 'personalBank',
						name: 'personalBank',
						component: resolve => require(['@/view/evaluation/bank/index.vue'], resolve)
					},
					{
						path: 'createCompose',
						name: 'createCompose',
						component: resolve => require(['@/view/evaluation/index/CreateCompose.vue'], resolve)
					}
				]
			},
			{
				path: 'createPaper',
				name: 'createPaper',
				component: resolve => require(['@/view/evaluation/index/CreatePaper.vue'], resolve)
			},
			{
				path: 'myCourse',
				name: 'myCourse',
				component: resolve => require(['@/view/newcourse/MyCourse.vue'], resolve)
			},
			{
				path: 'CourseTable',
				name: 'CourseTable',
				component: resolve => require(['@/view/newcourse/CourseTable.vue'], resolve)
			},

			//学校课程管理
			{
				path: 'ManageCourse',
				name: 'ManageCourse',
				component: resolve => require(['@/view/newcourse/ManageCourse.vue'], resolve)
			},
			//课程时间管理
			{
				path: 'CourseTime',
				name: 'CourseTime',
				component: resolve => require(['@/view/newcourse/CourseTime.vue'], resolve)
			},

			//排课管理新版
			{
				path: 'NewCoursePlan',
				name: 'NewCoursePlan',
				component: resolve => require(['@/view/newcourse/NewCoursePlan.vue'], resolve)
			},
			// 新课纲管理
			{
				path: 'syllabus',
				name: 'syllabus',
				component: resolve => require(['@/view/syllabus/newSyllabus/Index.vue'], resolve)
			},
			// 新课纲管理
			{
				path: 'personalSyllabus',
				name: 'personalSyllabus',
				component: resolve => require(['@/view/syllabus/newSyllabus/Index.vue'], resolve)
			},
			//校本资源管理
			{
				path: 'schoolcontent',
				name: 'schoolcontent',
				component: resolve => require(['@/view/teachcontent/index.vue'], resolve)
			},
			//个人内容管理
			{
				path: 'personalcontent',
				name: 'personalcontent',
				component: resolve => require(['@/view/teachcontent/index.vue'], resolve)
			},
			// 知识点管理
			{
				path: 'knowledge',
				name: 'knowledge',
				component: resolve => require(['@/view/knowledge-point/index/Index.vue'], resolve)
			},
			// 新学习活动模块
			{
				path: 'createEvaluation',
				name: 'createEvaluation',
				component: resolve => require(['@/view/learnactivity/CreateEvaluation.vue'], resolve)
			},
			//创建学习单元
			{
				path: 'createLearnUnit',
				name: 'createLearnUnit',
				component: resolve => require(['@/view/selfstudy/CreateLearnUnit.vue'], resolve)
			},
			//创建作业活动
			//{
			//    path: 'createHomeWork',
			//    name: 'createHomeWork',
			//    component: resolve => require(['@/view/selflearning/CreateHomeWork.vue'], resolve)
			//},
			//创建编序式学习
			{
				path: 'createOrderLearn',
				name: 'createOrderLearn',
				component: resolve => require(['@/view/selfstudy/CreateOrderLearn.vue'], resolve)
			},
			//管理评测页面(学校)
			{
				path: 'schoolEvaluation',
				name: 'schoolEvaluation',
				component: resolve => require(['@/view/learnactivity/ManageEvaluation.vue'], resolve)
			},
			//管理评测页面（班级）
			{
				path: 'classEvaluation',
				name: 'classEvaluation',
				component: resolve => require(['@/view/learnactivity/ManageEvaluation.vue'], resolve)
			},
			//管理活动记录页面
			{
				path: 'manageRecord',
				name: 'manageRecord',
				component: resolve => require(['@/view/learnactivity/ManageRecord.vue'], resolve)
			},
			//管理问卷调查
			{
				path: 'manageQuestionnaire',
				name: 'manageQuestionnaire',
				component: resolve => require(['@/view/questionnaire/ManageQuestionnaire.vue'], resolve)
			},
			//管理问卷调查
			{
				path: 'personalSurvey',
				name: 'personalSurvey',
				component: resolve => require(['@/view/questionnaire/ManageQuestionnaire.vue'], resolve)
			},
			//新版自主学习
			{
				path: 'SelfLearn',
				name: 'selfLearn',
				component: resolve => require(['@/view/selfstudy/SelfLearn.vue'], resolve)
			},
			{
				path: 'manageHomeWork',
				name: 'manageHomeWork',
				component: resolve => require(['@/view/homework/ManageHomeWork.vue'], resolve)
			},
			{
				path: 'manageVote',
				name: 'manageVote',
				component: resolve => require(['@/view/vote/ManageVote.vue'], resolve)
			},
			{
				path: 'personalVote',
				name: 'personalVote',
				component: resolve => require(['@/view/vote/ManageVote.vue'], resolve)
			},
			//主页
			{
				path: 'homePage',
				name: 'homePage',
				component: resolve => require(['@/view/homepage/HomePage.vue'], resolve)
			},
			//班级管理
			{
				path: 'manageClass',
				name: 'manageClass',
				component: resolve => require(['@/view/classmgt/ManageClass.vue'], resolve)
			},
			//用户反馈
			{
				path: 'feedback',
				name: 'feedback',
				component: resolve => require(['@/view/feedback/Feedback.vue'], resolve)
			},
			//设置
			{
				path: 'settings',
				name: 'settings',
				component: resolve => require(['@/view/settings/Index.vue'], resolve)
			},
			//课堂记录
			{
				path: 'classRecord',
				name: 'classRecord',
				component: resolve => require(['@/view/classrecord/ClassRecord.vue'], resolve)
			},

			//Louise的智能教室使用分析儀表板
			{
				path: 'scboard',
				name: 'scboard',
				component: resolve => require(['@/view/scboard/Index.vue'], resolve)
			},

			//Louise的教師儲存空間授權管理分頁
			{
				path: 'serviceDriveAuth',
				name: 'serviceDriveAuth',
				component: resolve => require(['@/view/serviceDriveAuth/Index.vue'], resolve)
			}


			////排课管理（废弃）
			//{
			//    path: 'CoursePlan',
			//    name: 'CoursePlan',
			//    component: resolve => require(['@/view/newcourse/CoursePlan.vue'], resolve)
			//},
		]
	},
	//学生端
	{
		name: 'studentWeb',
		path: '/studentWeb',
		redirect: '/studentWeb/homeView',
		component: resolve => require(['@/view/student-web/App'], resolve),
		meta: {
			// 暫時連其他權限也可以登入 日後要改成只有學生
			middleware: ['login', 'role:admin|teacher|student'], // 强制登录（会自动跳转到登录页），角色为管理员或教师
		},
		children: [{
			name: "homeView",
			path: "homeView",
				component: resolve => require(['@/components/student-web/HomeView/HomeView'], resolve),
			},
			{
				name: "courseList",
				path: "courseList",
				component: resolve => require(['@/components/student-web/HomeView/CourseListView'], resolve), //是元件不用''
			},
			{
				name: "setting",
				path: "setting",
				component: resolve => require(['@/components/student-web/SettingView/SettingView'], resolve), //是元件不用'
			},
			{
				name: "eventView",
				path: "eventView",
				component: resolve => require(['@/components/student-web/EventView/EventView'], resolve),
			},
			{
				name: "eventView/:name",
				path: "eventView/:name",
				component: resolve => require(['@/components/student-web/EventView/EventView'], resolve),
			},
			{
				name: "eventLatest",
				path: "eventLatest",
				component: resolve => require(['@/components/student-web/EventView/EventOverView/EventLatest'], resolve),
			},
			{
				name: "studyView",
				path: "studyView",
				component: resolve => require(['@/components/student-web/StudyView/StudyView'], resolve),
			},
			{
				name: "hiteachView",
				path: "hiteachView",
				component: resolve => require(['@/components/student-web/HiteachView/HiteachView'], resolve),
			},
			{
				name: "informView",
				path: "informView",
				component: resolve => require(['@/components/student-web/InformView/InformView'], resolve),
			}
		]
	}
]
