import $api from '@/api'
import store from '@/store'
import $tools from './public.js'


export default {
	/* 获取试题保存在Blob的JSON文件 */
	createBlobItem(item){
		return new Promise((r,j) => {
			let itemJson = {
				id:item.id,
				exercise:{
					answer:item.answer,
					explain:item.explain,
					type:item.type,
					points:item.points,
					field:item.field,
					level:item.level,
					periodId:item.periodId,
					gradeIds:item.gradeIds,
					subjectId:item.subjectId,
					children:item.children || [],
					scope:item.scope,
					score: 0,
					repairResource:item.repairResource,
				},
				render:2,
				item:[{
					question:item.question,
					option:item.option
				}]
			}
			r(itemJson)
		})
	},
	
	/* 获取保存在COSMOS里面的试题格式 */
	createCosmosItem(item){
		return new Promise((r,j) => {
			let cosmosItem  = {
				id:item.id,
				code:item.code,
				scope:item.scope,
				score:0,
				type:item.type,
				question:this.getSimpleText(item.question),
				points:item.points,
				field:item.field,
				level:item.level,
				periodId:item.periodId,
				gradeIds:item.gradeIds,
				subjectId:item.subjectId,
				blob:item.blob
			}
			r(cosmosItem)
		})
	},
	
	/* 生成试卷的index.json文件格式 */
	createBlobPaper(paper,urls){
		return new Promise((r,j) => {
			let paperItem  = {
				id:paper.id,
				name:paper.name,
				code:paper.code,
				scope:paper.scope,
				multipleRule:paper.multipleRule,
				urls:urls,
				scoring:paper.scoring,
				points:paper.points,
				periodId:paper.periodId,
				gradeIds:paper.gradeIds,
				subjectId:paper.subjectId,
				subjectName:paper.subjectName,
				score:paper.score
			}
			r(paperItem)
		})
	},
	
	/* 生成试卷保存在cosmos的数据结构 */
	createCosmosPaper(paper){
		return new Promise((r,j) => {
			let paperItem  = {
				id:paper.id,
				name:paper.name,
				code:paper.code,
				blob:paper.blob,
				scope:paper.scope,
				scoring:paper.scoring,
				points:paper.points,
				periodId:paper.periodId,
				gradeIds:paper.gradeIds,
				subjectId:paper.subjectId,
				subjectName:paper.subjectName,
				score:paper.score,
				multipleRule:paper.multipleRule
			}
			r(paperItem)
		})
	},
	
	/* 获取完整的试题数据 */
	getFullItem(list){
		return new Promise(async (resolve,reject) => {
			let promiseArr = []
			for(let i=0; i<list.length; i++){
				promiseArr.push(new Promise(async (r,j) => {
					if(list[i].blob){
						const blobHost = list[i].scope === 'school' ?  JSON.parse(decodeURIComponent(localStorage.school_profile, "utf-8")).blob_uri :  JSON.parse(decodeURIComponent(localStorage.user_profile, "utf-8")).blob_uri
						// 根据试题的Blob地址 去读取JSON文件
						let sasString = list[i].scope === 'school' ?  await $tools.getSchoolSas() : await $tools.getPrivateSas()
						let jsonInfo = await $tools.getFile(blobHost + list[i].blob + sasString.sas)
						let jsonData = JSON.parse(jsonInfo)
						// 调整渲染试题数据结构
						jsonData.id = list[i].id
						jsonData.exercise.question = jsonData.item[0].question
						jsonData.exercise.blob = list[i].blob
						jsonData.exercise.code = list[i].code
						jsonData.exercise.option = jsonData.item[0].option
						jsonData.exercise.id = list[i].id
						r(jsonData.exercise)
					}else{
						r(null)
					}
				}))
			}
			Promise.all(promiseArr).then(result => {
				resolve(result)
			})
		})
	},
	
	/* 获取完整的试卷数据 */
	getFullPaper(paper){
		return new Promise(async (r,j) => {
			let blobHost = paper.scope === 'school' ?  JSON.parse(decodeURIComponent(localStorage.school_profile, "utf-8")).blob_uri :  JSON.parse(decodeURIComponent(localStorage.user_profile, "utf-8")).blob_uri
			// 根据试卷的Blob地址 去读取JSON文件
			let sasString = paper.scope === 'school' ?  await $tools.getSchoolSas() : await $tools.getPrivateSas()
			try{
				let jsonInfo = await $tools.getFile(blobHost + paper.blob + '/index.json' + sasString.sas)
				let jsonData = JSON.parse(jsonInfo)
				// 获取试卷包含的试题数据并包装好
				if(jsonData.urls && jsonData.urls.length){
					jsonData.item = []
					const path = blobHost + paper.blob
					jsonData.urls.forEach(async (itemUrl,index) => {
						// 获取题目JSON并且包装成完整试题对象
						let itemJson = JSON.parse(await $tools.getFile(path + '/' + itemUrl + sasString.sas))
						itemJson.exercise.question = itemJson.item[0].question
						itemJson.exercise.option = itemJson.item[0].option
						itemJson.exercise.id = itemJson.id 
						itemJson.exercise.score = jsonData.scoring[index].score
						jsonData.item.push(itemJson.exercise)
					})
				}
				r(jsonData)
			}catch(e){
				j(e)
			}
		})
	},
	
	
	/* 获取完整的试卷数据 */
	getStuPaper(paper){
		return new Promise(async (r,j) => {
			let blobHost = JSON.parse(decodeURIComponent(localStorage.student_profile, "utf-8")).blob_uri
			let splitHost = blobHost.split('/')
			// 根据试卷的Blob地址 去读取JSON文件
			let sasString = paper.scope === 'school' ?  await $tools.getSchoolSas(paper.code) : await $tools.getPrivateSas(paper.code)
			try{
				let jsonInfo = await $tools.getFile(sasString.url + '/' + paper.code + paper.blob + '/index.json' + sasString.sas)
				let jsonData = JSON.parse(jsonInfo)
				// 获取试卷包含的试题数据并包装好
				if(jsonData.urls && jsonData.urls.length){
					jsonData.item = []
					const path = sasString.url + '/' + paper.code + paper.blob
					jsonData.urls.forEach(async (itemUrl,index) => {
						// 获取题目JSON并且包装成完整试题对象
						let itemJson = JSON.parse(await $tools.getFile(path + '/' + itemUrl + sasString.sas))
						itemJson.exercise.question = itemJson.item[0].question
						itemJson.exercise.option = itemJson.item[0].option
						itemJson.exercise.id = itemJson.id 
						itemJson.exercise.score = jsonData.scoring[index].score
						jsonData.item.push(itemJson.exercise)
					})
				}
				r(jsonData)
			}catch(e){
				j(e)
			}
		})
	},
	
	/* 提取富文本内容中的文本 */
	getSimpleText(html) {
		// var msg = html.replace(/<(?!img).*?>/g, '') // 执行替换成空字符
		var r = /<[^>]*>|/g;
		return html.replace(r, '')
	},
}
