using Microsoft.AspNetCore.Hosting;
using Microsoft.AspNetCore.Mvc;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;

using TEAMModelOS.Helper.Common.FileHelper;
using TEAMModelOS.SDK;
using TEAMModelOS.SDK.Helper.Common.JsonHelper;

namespace TEAMModelOS.Starter.CloudServer.Controllers.AnalysisControllers
{
    [Route("api/[controller]")]
    [ApiController]
    public class ClassController: Controller
    {
        IWebHostEnvironment _hostingEnvironment;

        public ClassController(IWebHostEnvironment hostingEnvironment)
        {
            _hostingEnvironment = hostingEnvironment;
        }
        /// <summary>
        /// 获取年级菜单
        /// </summary>
        /// <param name="identity"></param>
        /// <returns></returns>
        [HttpGet("getGrade")]
        public BaseResponse getGrade(string identity)
        {
            string contentRootPath = _hostingEnvironment.ContentRootPath;
            //identity = "grade";
            ResponseBuilder builder = ResponseBuilder.custom();
            string data = FileTool.getJson(contentRootPath,identity);
            builder.Data(JsonConvert.DeserializeObject(data));
            return builder.build();
        }

        /// <summary>
        /// 获取学期菜单
        /// </summary>
        /// <param name="identity"></param>
        /// <returns></returns>
        [HttpGet("getTerm")]
        public BaseResponse getTerm(string identity)
        {
            string contentRootPath = _hostingEnvironment.ContentRootPath;
            //identity = "term";
            ResponseBuilder builder = ResponseBuilder.custom();
            string data = FileTool.getJson(contentRootPath, identity);
            builder.Data(JsonConvert.DeserializeObject(data));
            return builder.build();
        }
        /// <summary>
        /// 获取考试列表
        /// </summary>
        /// <param name="identity"></param>
        /// <returns></returns>
        [HttpGet("getExam")]
        public BaseResponse getExam(string identity)
        {
            string contentRootPath = _hostingEnvironment.ContentRootPath;
            //identity = "Exam";
            ResponseBuilder builder = ResponseBuilder.custom();
            string data = FileTool.getJson(contentRootPath, identity);
            builder.Data(JsonConvert.DeserializeObject(data));
            return builder.build();
        }

        /// <summary>
        /// 获取基础信息
        /// </summary>
        /// <param name="identity"></param>
        /// <returns></returns>
        [HttpGet("getBase")]
        public BaseResponse getBase(string identity)
        {
            string contentRootPath = _hostingEnvironment.ContentRootPath;
            identity = "Base";
            ResponseBuilder builder = ResponseBuilder.custom();
            string data = FileTool.getJson(contentRootPath, identity);
            builder.Data(JsonConvert.DeserializeObject(data));
            return builder.build();
        }
        /// <summary>
        /// 点击关注年级后，动态数据
        /// </summary>
        /// <param name="identity"></param>
        /// <returns></returns>
        [HttpGet("getChange")]
        public BaseResponse getChange(string identity)
        {
            string contentRootPath = _hostingEnvironment.ContentRootPath;
            identity = "Changegrade";
            ResponseBuilder builder = ResponseBuilder.custom();
            string data = FileTool.getJson(contentRootPath, identity);
            builder.Data(JsonConvert.DeserializeObject(data));
            return builder.build();
        }

        /// <summary>
        /// 点击关注学年期后，动态数据
        /// </summary>
        /// <param name="identity"></param>
        /// <returns></returns>
        [HttpGet("getChangeterm")]
        public BaseResponse getChangeterm(string identity)
        {
            string contentRootPath = _hostingEnvironment.ContentRootPath;
            identity = "Changeterm";
            ResponseBuilder builder = ResponseBuilder.custom();
            string data = FileTool.getJson(contentRootPath, identity);
            builder.Data(JsonConvert.DeserializeObject(data));
            return builder.build();
        }

        /// <summary>
        /// 点击某次考试详情，动态数据
        /// </summary>
        /// <param name="identity"></param>
        /// <returns></returns>
        [HttpGet("getChangeExam")]
        public BaseResponse getChangeExam(string identity)
        {
            string contentRootPath = _hostingEnvironment.ContentRootPath;
            identity = "ChangeExam";
            ResponseBuilder builder = ResponseBuilder.custom();
            string data = FileTool.getJson(contentRootPath, identity);
            builder.Data(JsonConvert.DeserializeObject(data));
            return builder.build();
        }



        /// <summary>
        /// 筛选考试类型，动态数据
        /// </summary>
        /// <param name="identity"></param>
        /// <returns></returns>
        [HttpGet("getChangeExamType")]
        public BaseResponse getChangeExamType(string identity)
        {
            string contentRootPath = _hostingEnvironment.ContentRootPath;
            identity = "ChangeExamType";
            ResponseBuilder builder = ResponseBuilder.custom();
            string data = FileTool.getJson(contentRootPath, identity);
            builder.Data(JsonConvert.DeserializeObject(data));
            return builder.build();
        }

        /// <summary>
        /// 显示文科按钮，动态数据
        /// </summary>
        /// <param name="identity"></param>
        /// <returns></returns>
        [HttpGet("getChangeArts")]
        public BaseResponse getChangeArts(string identity)
        {
            string contentRootPath = _hostingEnvironment.ContentRootPath;
            identity = "Changearts";
            ResponseBuilder builder = ResponseBuilder.custom();
            string data = FileTool.getJson(contentRootPath, identity);
            builder.Data(JsonConvert.DeserializeObject(data));
            return builder.build();
        }

        /// <summary>
        /// 显示理科按钮，动态数据
        /// </summary>
        /// <param name="identity"></param>
        /// <returns></returns>
        [HttpGet("getChangeScience")]
        public BaseResponse getChangeScience(string identity)
        {
            string contentRootPath = _hostingEnvironment.ContentRootPath;
            identity = "Changescience";
            ResponseBuilder builder = ResponseBuilder.custom();
            string data = FileTool.getJson(contentRootPath, identity);
            builder.Data(JsonConvert.DeserializeObject(data));
            return builder.build();
        }


        [HttpGet("getClass")]
        public BaseResponse getClass(string identity)
        {
            string contentRootPath = _hostingEnvironment.ContentRootPath;
            identity = "json";
            ResponseBuilder builder = ResponseBuilder.custom();
            string data = FileTool.getJson(contentRootPath, identity);
            builder.Data(data);
            return builder.build();
        }
    }
}
