using Azure.Cosmos;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Mvc;
using Microsoft.CodeAnalysis.Options;
using Microsoft.Extensions.Options;
using System;
using System.Collections.Generic;
using System.Dynamic;
using System.IdentityModel.Tokens.Jwt;
using System.IO;
using System.Linq;
using System.Text;
using System.Text.Json;
using System.Threading.Tasks;
using TEAMModelOS.Models;
using TEAMModelOS.Models.CommonInfo;
using TEAMModelOS.Models.SchoolInfo;
using TEAMModelOS.Models.StudentInfo;
using TEAMModelOS.SDK;
using TEAMModelOS.SDK.Context.Constant.Common;
using TEAMModelOS.SDK.DI;
using TEAMModelOS.SDK.DI.AzureCosmos.Inner;
using TEAMModelOS.SDK.Extension;
using TEAMModelOS.SDK.Helper.Common.CollectionHelper;
using TEAMModelOS.SDK.Helper.Common.StringHelper;

namespace TEAMModelOS.Controllers
{
    [ProducesResponseType(StatusCodes.Status200OK)]
    [ProducesResponseType(StatusCodes.Status400BadRequest)]
    //[Authorize(Roles = "IES5")
    [Route("school/exam")]
    [ApiController]
    public class ExamController : ControllerBase
    {
        private readonly AzureCosmosFactory _azureCosmos;        
        private readonly SnowflakeId _snowflakeId;     
        private readonly AzureServiceBusFactory _serviceBus;
        private readonly DingDing _dingDing;
        private readonly Option _option;

        public ExamController(AzureCosmosFactory azureCosmos, AzureServiceBusFactory serviceBus, SnowflakeId snowflakeId,
                        DingDing dingDing,
            IOptionsSnapshot<Option> option)
        {            
            _azureCosmos = azureCosmos;
            _serviceBus = serviceBus;
            _snowflakeId = snowflakeId;
            _dingDing = dingDing;
            _option = option?.Value;
        }

        /// <summary>
        /// 保存考试信息
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        [ProducesDefaultResponseType]
        //[AuthToken(Roles = "Teacher")]
        [HttpPost("save")]
        public async Task<IActionResult> Save(ExamInfo request)
        {
            //新增
            //string code = request.code;
            var client = _azureCosmos.GetCosmosClient();
            ExamInfo exam;
            request.ttl = -1;
            request.pk = "Exam";
            request.code = "Exam-" + request.code;
            request.createTime = DateTimeOffset.UtcNow.ToUnixTimeMilliseconds();
            /*            if (request.publish.Equals("0"))
                        {
                            request.progress = "going";
                        }
                        else if (request.publish.Equals("1"))
                        {

                            //string msgId = _snowflakeId.NextId() + "";
                            request.progress = "finish";
                            *//*long SequenceNumber = await _serviceBus.GetServiceBusClient().SendLeamMessage<ExamInfo>(Constants.TopicName, request.id, request.code, request.startTime, 200, msgId);
                            request.sequenceNumber = SequenceNumber;*//*

                        }*/
            if (string.IsNullOrEmpty(request.id))
            {
                request.id = Guid.NewGuid().ToString();
                request.progress = "pending";
                int count = 0;
                for (int i =0; i < request.targetClassIds.Count;i++) {
                    var sresponse = await client.GetContainer("TEAMModelOS", "School").ReadItemStreamAsync(request.targetClassIds[i], new PartitionKey($"{request.code}"));
                    if (sresponse.Status == 200) {
                        using var json = await JsonDocument.ParseAsync(sresponse.ContentStream);
                        Classroom classroom = json.ToObject<Classroom>();
                        count += classroom.students.Count;
                    }
                }
                request.stuCount = count;
                long SequenceNumber = await _serviceBus.GetServiceBusClient().SendLeamMessage<ExamInfo>(Constants.TopicName, request.id, request.code, request.startTime);
                request.sequenceNumber = SequenceNumber;
                exam = await client.GetContainer("TEAMModelOS", "Common").CreateItemAsync(request, new PartitionKey($"{request.code}"));
                //await _serviceBus.GetServiceBusClient().SendLeamMessage<ExamInfo>(Constants.TopicName, request.id, request.code, request.startTime);
                //request.sequenceNumber = SequenceNumber;
            }
            else
            {
                ExamInfo info = await client.GetContainer("TEAMModelOS", "Common").ReadItemAsync<ExamInfo>(request.id, new PartitionKey($"{request.code}"));
                if (info.progress.Equals("going"))
                {
                    return Ok(new { v = "活动正在进行中" });
                }
                request.progress = info.progress;
                await _serviceBus.GetServiceBusClient().cancelMessage(Constants.TopicName, info.sequenceNumber);
                long SequenceNumber = await _serviceBus.GetServiceBusClient().SendLeamMessage<ExamInfo>(Constants.TopicName, request.id, request.code, request.startTime);
                request.sequenceNumber = SequenceNumber;
                exam = await client.GetContainer("TEAMModelOS", "Common").ReplaceItemAsync(request, request.id, new PartitionKey($"{request.code}"));
                //await _serviceBus.GetServiceBusClient().SendLeamMessage<ExamInfo>(Constants.TopicName, request.id, request.code, request.startTime);
            }
            //Survey homeWork = await _azureCosmos.SaveOrUpdate<Survey>(request.survey);

            //设定结束时间
            //string msgEndId = _snowflakeId.NextId() + "";
            await _serviceBus.GetServiceBusClient().SendLeamMessage<ExamInfo>(Constants.TopicName, request.id, request.code, request.endTime);
            return Ok(new{ exam });
        }
        /// <summary>
        /// 删除
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        [ProducesDefaultResponseType]
        //[AuthToken(Roles = "Teacher")]
        [HttpPost("delete")]
        public async Task<IActionResult> Delete(JsonElement request)
        {
            /* ResponseBuilder builder = ResponseBuilder.custom();
             IdPk items = await _azureCosmos.DeleteAsync<ExamInfo>(request.id, request.pk);
             await _azureCosmos.DeleteAll<Paper>(new Dictionary<string, object>() { { "code", request.id } });
             return builder.Data(items).build();*/
            try
            {
                if (!request.TryGetProperty("id", out JsonElement id)) return BadRequest();
                if (!request.TryGetProperty("code", out JsonElement code)) return BadRequest();
                //if (!request.TryGetProperty("scope", out JsonElement scope)) return BadRequest();
                var client = _azureCosmos.GetCosmosClient();
                string school_code = code.ToString().Substring(typeof(ExamClassResult).Name.Length + 1);
                var response = await client.GetContainer("TEAMModelOS", "Common").DeleteItemStreamAsync(id.ToString(), new PartitionKey($"{code}"));
                List<ExamClassResult> examClassResults = new List<ExamClassResult>();
                await foreach (var item in client.GetContainer("TEAMModelOS", "Common").GetItemQueryStreamIterator(queryText: $"select value(c) from c where c.examId = '{id}'", requestOptions: new QueryRequestOptions() { PartitionKey = new PartitionKey($"ExamClassResult-{school_code}") }))
                {
                    using var json = await JsonDocument.ParseAsync(item.ContentStream);
                    if (json.RootElement.TryGetProperty("_count", out JsonElement count) && count.GetUInt16() > 0)
                    {
                        foreach (var obj in json.RootElement.GetProperty("Documents").EnumerateArray())
                        {
                            examClassResults.Add(obj.ToObject<ExamClassResult>());
                        }
                    }
                }
                await _azureCosmos.DeleteAll<ExamClassResult>(examClassResults);
                return Ok(new { id });



            }
            catch (Exception e)
            {
                return BadRequest();
            }
        }
        /// <summary>
        /// 查询考试信息
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        [ProducesDefaultResponseType]
        //[AuthToken(Roles = "Teacher")]
        [HttpPost("find")]
        public async Task<IActionResult> Find(JsonElement requert)
        {
            //ResponseBuilder builder = ResponseBuilder.custom();
            //if (!requert.TryGetProperty("id_token", out JsonElement id_token)) return BadRequest();
            if (!requert.TryGetProperty("code", out JsonElement school_code)) return BadRequest();
            //if (!requert.TryGetProperty("scope", out JsonElement scope)) return BadRequest();
            //var jwt = new JwtSecurityToken(id_token.GetString());
            //if (!jwt.Payload.Iss.Equals("account.teammodel", StringComparison.Ordinal)) return BadRequest();
            //var id = jwt.Payload.Sub;
            var client = _azureCosmos.GetCosmosClient();
            List<ExamInfo> examInfo = new List<ExamInfo>();
            var query = $"select c.id,c.name,c.code,c.startTime,c.stuCount,c.type,c.progress,c.examType from c ";
            await foreach (var item in client.GetContainer("TEAMModelOS", "Common").GetItemQueryStreamIterator(queryText: query, requestOptions: new QueryRequestOptions() { PartitionKey = new PartitionKey($"Exam-{school_code}") }))
                {
                    using var json = await JsonDocument.ParseAsync(item.ContentStream);
                    if (json.RootElement.TryGetProperty("_count", out JsonElement count) && count.GetUInt16() > 0)
                    {
                        foreach (var obj in json.RootElement.GetProperty("Documents").EnumerateArray())
                        {
                            //obj.TryGetProperty("progress", out JsonElement progress);
                            examInfo.Add(obj.ToObject<ExamInfo>());
                        }
                    }
                }
            return Ok(new { examInfo });
            /*if (StringHelper.getKeyCount(request) > 0)
            {
                return builder.Data(await _azureCosmos.FindByDict<ExamInfo>(request)).build();
            }
            else {
                return builder.build();
            }*/
        }
        [ProducesDefaultResponseType]
        //[AuthToken(Roles = "Teacher")]
        [HttpPost("find-summary")]
        public async Task<IActionResult> FindSummary(JsonElement requert)
        {
            //ResponseBuilder builder = ResponseBuilder.custom();
            //if (!requert.TryGetProperty("id_token", out JsonElement id_token)) return BadRequest();
            if (!requert.TryGetProperty("id", out JsonElement id)) return BadRequest();
            if (!requert.TryGetProperty("code", out JsonElement code)) return BadRequest();
            //if (!requert.TryGetProperty("scope", out JsonElement scope)) return BadRequest();
            //var jwt = new JwtSecurityToken(id_token.GetString());
            //if (!jwt.Payload.Iss.Equals("account.teammodel", StringComparison.Ordinal)) return BadRequest();
            //var id = jwt.Payload.Sub;
            var client = _azureCosmos.GetCosmosClient();
            List<ExamInfo> examInfo = new List<ExamInfo>();
            //var query = $"select value(c) from c ";
            await foreach (var item in client.GetContainer("TEAMModelOS", "Common").GetItemQueryStreamIterator(queryText: $"select value(c) from c where c.id = '{id}'", requestOptions: new QueryRequestOptions() { PartitionKey = new PartitionKey($"{code}") }))
            {
                using var json = await JsonDocument.ParseAsync(item.ContentStream);
                if (json.RootElement.TryGetProperty("_count", out JsonElement count) && count.GetUInt16() > 0)
                {
                    foreach (var obj in json.RootElement.GetProperty("Documents").EnumerateArray())
                    {
                        //obj.TryGetProperty("progress", out JsonElement progress);
                        examInfo.Add(obj.ToObject<ExamInfo>());
                    }
                }
            }
            return Ok(new { examInfo });
        }
        /// <summary>
        /// 教师阅卷
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        [ProducesDefaultResponseType]
        //[AuthToken(Roles = "Teacher")]
        [HttpPost("marking")]
        public async Task<BaseResponse> Marking(ExamRecord request)
        {
            ResponseBuilder builder = ResponseBuilder.custom();
            //判断是否每一个题目都有分数
            List<ExamInfo> exams = await _azureCosmos.FindByDict<ExamInfo>(new Dictionary<string, object> { { "id", request.examCode } });

            if (exams.IsNotEmpty())
            {
                ExamInfo examInfo = exams[0];
                //提交答案时间必须是状态已发布，且时间在起止时间内
                if ( examInfo.status == 300 &&
                   examInfo.endTime <= new DateTimeOffset(DateTime.UtcNow).ToUnixTimeMilliseconds())
                {
                    return builder.Data(await _azureCosmos.SaveOrUpdate(request)).build();
                }
                else
                {
                    return builder.Error(ResponseCode.FAILED, "请在作答时间段内提交答案!").build();
                }
            }
            else
            {
                return builder.Error(ResponseCode.DATA_EXIST, "考试不存在!").build();
            }
        }


        /// <summary>
        /// 学生回答问题
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        [ProducesDefaultResponseType]
        //[AuthToken(Roles = "Teacher")]
        [HttpPost("upsert-all-record")]
        public async Task<BaseResponse> upsertRecord(List<ExamRecord> request)
        {
            ResponseBuilder builder = ResponseBuilder.custom();
            if (request.IsNotEmpty())
            {
                //要先处理状态，判断卷子是否存在，并判断卷子归属的考试是否允许再次提交
                List<ExamInfo> exams = await _azureCosmos.FindByDict<ExamInfo>(new Dictionary<string, object> { { "id", request[0].examCode } });
                if (exams.IsNotEmpty())
                {
                    
                    return builder.Data(await _azureCosmos.SaveOrUpdateAll(request)).build();
                }
                else
                {
                    return builder.Error(ResponseCode.DATA_EXIST, "考试不存在!").build();
                    
                }
            }
            else {
                return builder.Error(ResponseCode.PARAMS_ERROR, "作答数据为空!").build();
            }
        }
        /// <summary>
        /// 学生回答问题
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        [ProducesDefaultResponseType]
        //[AuthToken(Roles = "Teacher")]
        [HttpPost("upsert-record")]
        public async Task<IActionResult> upsertRecord(JsonElement request)
        {
            if (!request.TryGetProperty("id", out JsonElement id)) return BadRequest();
            //if (!request.TryGetProperty("code", out JsonElement code)) return BadRequest();
            if (!request.TryGetProperty("answer", out JsonElement answer)) return BadRequest();
            if (!request.TryGetProperty("studentId", out JsonElement studentId)) return BadRequest();
            if (!request.TryGetProperty("subjectId", out JsonElement subjectId)) return BadRequest();
            if (!request.TryGetProperty("classId", out JsonElement classId)) return BadRequest();
            if (!request.TryGetProperty("multipleRule", out JsonElement multipleRule)) return BadRequest();
            //if (!request.TryGetProperty("answers ", out JsonElement tandardAnswer)) return BadRequest();
            if (!request.TryGetProperty("paperId", out JsonElement paperId)) return BadRequest();
            if (!request.TryGetProperty("school", out JsonElement school)) return BadRequest();
            var client = _azureCosmos.GetCosmosClient();         
            List<ExamClassResult> examClassResults = new List<ExamClassResult>();
            await foreach (var item in client.GetContainer("TEAMModelOS", "Common").GetItemQueryStreamIterator(
                queryText: $"select value(c) from c where c.examId = '{id}' and c.subjectId = '{subjectId}' and c.info.id = '{classId}'" ,
                requestOptions: new QueryRequestOptions() { PartitionKey = new PartitionKey($"ExamClassResult-{school}") }))
            {
                using var json = await JsonDocument.ParseAsync(item.ContentStream);
                if (json.RootElement.TryGetProperty("_count", out JsonElement count) && count.GetUInt16() > 0)
                {
                    foreach (var obj in json.RootElement.GetProperty("Documents").EnumerateArray())
                    {
                        examClassResults.Add(obj.ToObject<ExamClassResult>());
                    }
                }
            }
            ExamClassResult classResult;
            List<PaperSimple> standerAnswers = new List<PaperSimple>();
            await foreach (var item in client.GetContainer("TEAMModelOS", "Common").GetItemQueryStreamIterator(
                queryText: $"select A0.point,A0.answers from c join A0 in c.papers where c.id = '{id}'and A0.id = '{paperId}'",
                requestOptions: new QueryRequestOptions() { PartitionKey = new PartitionKey($"Exam-{school}") }))
            {
                using var json = await JsonDocument.ParseAsync(item.ContentStream);
                if (json.RootElement.TryGetProperty("_count", out JsonElement count) && count.GetUInt16() > 0)
                {
                    foreach (var obj in json.RootElement.GetProperty("Documents").EnumerateArray())
                    {
                        standerAnswers.Add(obj.ToObject<PaperSimple>());
                    }
                }
            }
            List<List<string>> ans = answer.ToObject<List<List<string>>>();
            List<List<string>> standard = standerAnswers[0].answers;
            List<double> points = standerAnswers[0].point;
            int index = examClassResults[0].studentIds.IndexOf(studentId.ToString());
            int rule = int.Parse(multipleRule.ToString());
            //classResult.studentAnswers[index] = ans;
            if (index == -1) {
                examClassResults[0].studentIds.Add(studentId.ToString());
                examClassResults[0].studentScores.Add(new List<double>());
                examClassResults[0].studentAnswers.Add(new List<List<string>>());
            }
            for (int i = 0;i < ans.Count;i++) {
                examClassResults[0].studentAnswers[index][i] = ans[i];
                //算分处理
                if (standard[i].Count > 0) {
                    if (ans[i].Count == standard[i].Count && standard[i].Count == 1)
                    {
                        foreach (string right in standard[i]) {
                            if (standard[i].Contains(right))
                            {
                                examClassResults[0].studentScores[index][i] = points[i];
                            }
                            else
                            {
                                examClassResults[0].studentScores[index][i] = 0;
                            }
                        }
                        
                    }
                    else {
                        if (rule > 0) {
                            int falseCount = 0;
                            foreach (string obj in ans[i])
                            {
                                if (!standard[i].Contains(obj))
                                {
                                    falseCount++;
                                }
                            }
                            switch (rule)
                            {
                                case 1:
                                    if (ans[i].Count == standard[i].Count)
                                    {
                                        if (falseCount == 0)
                                        {
                                            examClassResults[0].studentScores[index][i] = points[i];
                                        }
                                        else
                                        {
                                            examClassResults[0].studentScores[index][i] = 0;
                                        }
                                    }
                                    else
                                    {
                                        examClassResults[0].studentScores[index][i] = 0;
                                    }
                                    break;
                                case 2:
                                    if (falseCount > 0)
                                    {
                                        examClassResults[0].studentScores[index][i] = 0;
                                    }
                                    else
                                    {
                                        if (ans[i].Count == standard[i].Count)
                                        {
                                            examClassResults[0].studentScores[index][i] = points[i];
                                        }
                                        else
                                        {
                                            examClassResults[0].studentScores[index][i] = points[i] / 2;
                                        }

                                    }
                                    break;
                                case 3:
                                    if (falseCount > 0)
                                    {
                                        examClassResults[0].studentScores[index][i] = 0;
                                    }
                                    else
                                    {
                                        if (ans[i].Count == standard[i].Count)
                                        {
                                            examClassResults[0].studentScores[index][i] = points[i];
                                        }
                                        else
                                        {
                                            examClassResults[0].studentScores[index][i] = System.Math.Round(ans[i].Count / standard[i].Count * points[i], 1);
                                        }

                                    }
                                    break;
                                case 4:
                                    if (ans[i].Count == standard[i].Count)
                                    {
                                        examClassResults[0].studentScores[index][i] = points[i];
                                    }
                                    else
                                    {
                                        double persent = (standard[i].Count - 2 * falseCount) / standard[i].Count;
                                        if (persent <= 0)
                                        {
                                            examClassResults[0].studentScores[index][i] = 0;
                                        }
                                        else
                                        {
                                            examClassResults[0].studentScores[index][i] = System.Math.Round(persent * points[i], 1);
                                        }
                                    }
                                    break;
                            }                       
                        }
                    }
                }                               
            }
            classResult = await client.GetContainer("TEAMModelOS", "Common").ReplaceItemAsync(examClassResults[0], examClassResults[0].id, new PartitionKey($"{examClassResults[0].code}"));
            return Ok(new { classResult });
            /*ResponseBuilder builder = ResponseBuilder.custom();
            //要先处理状态，判断卷子是否存在，并判断卷子归属的考试是否允许再次提交
            List<ExamInfo> exams=  await _azureCosmos.FindByDict<ExamInfo>(new Dictionary<string, object> { { "id", request.examCode } });
            if (exams.IsNotEmpty())
            {
                ExamInfo examInfo = exams[0];
                //提交答案时间必须是状态已发布，且时间在起止时间内
                if (examInfo.startTime <= new DateTimeOffset(DateTime.UtcNow).ToUnixTimeMilliseconds() && examInfo.status == 200 &&
                   examInfo.endTime >= new DateTimeOffset(DateTime.UtcNow).ToUnixTimeMilliseconds())
                {
                    return builder.Data(await _azureCosmos.SaveOrUpdate(request)).build();
                }
                else
                {
                    return builder.Error(ResponseCode.FAILED, "请在作答时间段内提交答案!").build();
                }
            }
            else {
                return builder.Error(ResponseCode.DATA_EXIST, "考试不存在!").build();
            }*/
        }

        [ProducesDefaultResponseType]
        //[AuthToken(Roles = "Teacher")]
        [HttpPost("upsert-record-by-teacher")]
        public async Task<IActionResult> upsertRecordByTeacher(JsonElement request)
        {
            //ResponseBuilder builder = ResponseBuilder.custom();
            try {
                if (!request.TryGetProperty("id", out JsonElement id)) return BadRequest();
                if (!request.TryGetProperty("code", out JsonElement code)) return BadRequest();
                if (!request.TryGetProperty("point", out JsonElement point)) return BadRequest();
                if (!request.TryGetProperty("studentId", out JsonElement studentId)) return BadRequest();
                if (!request.TryGetProperty("classId", out JsonElement classId)) return BadRequest();
                if (!request.TryGetProperty("school", out JsonElement school)) return BadRequest();
                //要先处理状态，判断卷子是否存在，并判断卷子归属的考试是否允许再次提交
                //List<ExamInfo> exams = await _azureCosmos.FindByDict<ExamInfo>(new Dictionary<string, object> { { "id", request.examCode } });
                List<double> ans = point.ToObject<List<double>>();
                var client = _azureCosmos.GetCosmosClient();
                List<ExamClassResult> examClassResults = new List<ExamClassResult>();
                await foreach (var item in client.GetContainer("TEAMModelOS", "Common").GetItemQueryStreamIterator(
                    queryText: $"select value(c) from c where c.examId = '{id}' and c.subjectId = {studentId} and c.info.id = '{classId}'",
                    requestOptions: new QueryRequestOptions() { PartitionKey = new PartitionKey($"ExamClassResult-{school}") }))
                {
                    using var json = await JsonDocument.ParseAsync(item.ContentStream);
                    if (json.RootElement.TryGetProperty("_count", out JsonElement count) && count.GetUInt16() > 0)
                    {
                        foreach (var obj in json.RootElement.GetProperty("Documents").EnumerateArray())
                        {
                            examClassResults.Add(obj.ToObject<ExamClassResult>());
                        }
                    }
                }
                ExamClassResult classResult;
                //ExamInfo classResult = await client.GetContainer("TEAMModelOS", "Common").ReadItemAsync<ExamInfo>(id.ToString(), new PartitionKey($"{code}"));
                //ExamClassResult classResult = await client.GetContainer("TEAMModelOS", "Common").ReadItemAsync<ExamClassResult>(id.ToString(), new PartitionKey($"{code}"));
                /*foreach (double index in ans) {
                    classResult.studentScores.in
                }*/
                int index = examClassResults[0].studentIds.IndexOf(studentId.ToString());
                for (int i = 0;i < ans.Count;i++) {
                    examClassResults[0].studentScores[index][i] = ans[i];
                }
                classResult =  await client.GetContainer("TEAMModelOS", "Common").ReplaceItemAsync(examClassResults[0], examClassResults[0].id, new PartitionKey($"{examClassResults[0].code}"));
                return Ok( new { classResult });
            } catch (Exception ex) {
                await _dingDing.SendBotMsg($"CoreAPI2，{_option.Location}，exam/upsertRecordByTeacher()\n{ex.Message}", GroupNames.醍摩豆服務運維群組);
                return BadRequest();
            }
           
            /*if (exams.IsNotEmpty())
            {
                ExamInfo examInfo = exams[0];
                //提交答案时间必须是状态已发布，且时间在起止时间内
                if (examInfo.startTime <= new DateTimeOffset(DateTime.UtcNow).ToUnixTimeMilliseconds() && examInfo.status == 200 &&
                   examInfo.endTime >= new DateTimeOffset(DateTime.UtcNow).ToUnixTimeMilliseconds())
                {
                    return builder.Data(await _azureCosmos.SaveOrUpdate(request)).build();
                }
                else
                {
                    return builder.Error(ResponseCode.FAILED, "请在作答时间段内提交答案!").build();
                }
            }
            else
            {
                return builder.Error(ResponseCode.DATA_EXIST, "考试不存在!").build();
            }*/
        }

        /// <summary>
        /// 查询评测详细信息（教师或者学生通用）
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        [ProducesDefaultResponseType]
        //[AuthToken(Roles = "Teacher")]
        [HttpPost("find-summary-record")]
        public async Task<IActionResult> findSummaryRecord(JsonElement requert)
        {
            //ResponseBuilder builder = ResponseBuilder.custom();
            //var (id, school) = HttpContext.GetAuthTokenInfo();
            try {
                if (!requert.TryGetProperty("id", out JsonElement id)) return BadRequest();
                //if (!requert.TryGetProperty("code", out JsonElement school_code)) return BadRequest();
                if (!requert.TryGetProperty("subjectId", out JsonElement subjectId)) return BadRequest();
                if (!requert.TryGetProperty("school", out JsonElement school_code)) return BadRequest();
                // 如果只有学生id则返回学生参加过的考试 只返回相关摘要信息
                var client = _azureCosmos.GetCosmosClient();
                //string code = school_code.ToString().Substring(5);
                //ExamInfo info = await client.GetContainer("TEAMModelOS", "Common").ReadItemAsync<ExamInfo>(id.ToString(), new PartitionKey($"{school_code}"));
                List<object> examClassResults = new List<object>();                   
                await foreach (var item in client.GetContainer("TEAMModelOS", "Common").GetItemQueryStreamIterator(queryText: $"select c.id, c.code,c.info,c.studentIds,c.studentAnswers from c where c.examId = '{id}' and c.subjectId = '{subjectId}'", 
                    requestOptions: new QueryRequestOptions() { PartitionKey = new PartitionKey($"ExamClassResult-{school_code}") }))
                    {
                        using var json = await JsonDocument.ParseAsync(item.ContentStream);
                        if (json.RootElement.TryGetProperty("_count", out JsonElement count) && count.GetUInt16() > 0)
                        {
                            foreach (var obj in json.RootElement.GetProperty("Documents").EnumerateArray())
                            {
                                examClassResults.Add(obj.ToObject<object>());
                            }
                        }
                 }                
                /*if (StringHelper.getKeyCount(requert) == 1 && requert.TryGetProperty("code", out JsonElement code))
                {
                    List<object> props = new List<object>();
                    await foreach (var item in client.GetContainer("TEAMModelOS", "Student").GetItemQueryStreamIterator(queryText: $"select c.id, c.code, c.examCode, c.status ,c.mark, c.score from c where c.id = {id}", requestOptions: new QueryRequestOptions() { PartitionKey = new PartitionKey($"ExamRecord-{school_code}") }))
                    {
                        using var json = await JsonDocument.ParseAsync(item.ContentStream);
                        if (json.RootElement.TryGetProperty("_count", out JsonElement count) && count.GetUInt16() > 0)
                        {
                            foreach (var obj in json.RootElement.GetProperty("Documents").EnumerateArray())
                            {
                                props.Add(obj.ToObject<object>());
                            }
                        }
                    }
                    return Ok(new { props });
                }
                else
                {
                    if (requert.TryGetProperty("examCode", out JsonElement _))
                    {
                        //List<string> props = new List<string> { "id", "code", "examCode", "status", "mark", "score" };
                        //List<ExamRecord> examRecords = await _azureCosmos.FindByDict<ExamRecord>(request, props);
                        //return builder.Data(examRecords).Extend(new Dictionary<string, object> { { "props", props } }).build();
                        List<object> props = new List<object>();
                        await foreach (var item in client.GetContainer("TEAMModelOS", "Student").GetItemQueryStreamIterator(queryText: $"select c.id, c.code, c.examCode, c.status ,c.mark, c.score from c ", requestOptions: new QueryRequestOptions() { PartitionKey = new PartitionKey($"ExamRecord-{school_code}") }))
                        {
                            using var json = await JsonDocument.ParseAsync(item.ContentStream);
                            if (json.RootElement.TryGetProperty("_count", out JsonElement count) && count.GetUInt16() > 0)
                            {
                                foreach (var obj in json.RootElement.GetProperty("Documents").EnumerateArray())
                                {
                                    props.Add(obj.ToObject<object>());
                                }
                            }
                        }
                        return Ok(new { props });
                    }                   
                }*/
                return Ok(new { examClassResults });
            } catch (Exception ex) {
                await _dingDing.SendBotMsg($"CoreAPI2，{_option.Location}，exam/findSummaryRecord()\n{ex.Message}", GroupNames.醍摩豆服務運維群組);
                return BadRequest();
            }
            
        }
        //学生端查询评测列表
        [ProducesDefaultResponseType]
        //[AuthToken(Roles = "Student")]
        [HttpPost("find-summary-by-student")]
        public async Task<IActionResult> findSummaryStudent(JsonElement requert)
        {
            //ResponseBuilder builder = ResponseBuilder.custom();
            //var (id, school) = HttpContext.GetAuthTokenInfo();
            try
            {
                if (!requert.TryGetProperty("id", out JsonElement id)) return BadRequest();
                if (!requert.TryGetProperty("code", out JsonElement code)) return BadRequest();
                if (!requert.TryGetProperty("studentId", out JsonElement studentId)) return BadRequest();
                if (!requert.TryGetProperty("school", out JsonElement school)) return BadRequest();
                var client = _azureCosmos.GetCosmosClient();
                var query = $"select c.id,c.code,A0.id paperId,A0.code paperCode,A0.name paperName,A0.multipleRule,A0.scope,A0.blob from c join A0 in c.papers where c.id ='{id}' and c.progress = 'going'";
                List<object> props = new List<object>();
                await foreach (var item in client.GetContainer("TEAMModelOS", "Common").GetItemQueryStreamIterator(queryText:query, requestOptions: new QueryRequestOptions() { PartitionKey = new PartitionKey($"{code}") }))
                {
                    using var json = await JsonDocument.ParseAsync(item.ContentStream);
                    if (json.RootElement.TryGetProperty("_count", out JsonElement count) && count.GetUInt16() > 0)
                    {
                        foreach (var obj in json.RootElement.GetProperty("Documents").EnumerateArray())
                        {
                            props.Add(obj.ToObject<object>());
                        }
                    }
                }
                var querySubject = $"select c.subjects from c where c.id ='{id}' and c.progress = 'going'";
                //List<object> props = new List<object>();
                await foreach (var item in client.GetContainer("TEAMModelOS", "Common").GetItemQueryStreamIterator(queryText: querySubject, requestOptions: new QueryRequestOptions() { PartitionKey = new PartitionKey($"{code}") }))
                {
                    using var json = await JsonDocument.ParseAsync(item.ContentStream);
                    if (json.RootElement.TryGetProperty("_count", out JsonElement count) && count.GetUInt16() > 0)
                    {
                        foreach (var obj in json.RootElement.GetProperty("Documents").EnumerateArray())
                        {
                            props.Add(obj.ToObject<object>());
                        }
                    }
                }                
				var queryAnswers = $"select c.id,c.code,c.studentIds,c.subjectId,c.studentAnswers from c where c.examId ='{id}' and c.progress = 'going' and array_contains(c.studentIds,'{studentId}')";
				List<ExamClassResult> answers = new List<ExamClassResult>();
				await foreach (var item in client.GetContainer("TEAMModelOS", "Common").GetItemQueryStreamIterator(queryText: queryAnswers, 
                    requestOptions: new QueryRequestOptions() { PartitionKey = new PartitionKey($"ExamClassResult-{school}") }))
				{
					using var json = await JsonDocument.ParseAsync(item.ContentStream);
					if (json.RootElement.TryGetProperty("_count", out JsonElement count) && count.GetUInt16() > 0)
					{
						foreach (var obj in json.RootElement.GetProperty("Documents").EnumerateArray())
						{
							answers.Add(obj.ToObject<ExamClassResult>());
						}
					}
				}
                List<List<List<string>>> stuAns = new List<List<List<string>>>();
                if (answers.Count > 0) {                   
                    for (int i = 0; i < answers.Count; i++)
                    {
                        int index = answers[i].studentIds.IndexOf(studentId.ToString());
                        if (index == -1) {
                            break;
                        }
                        stuAns.Add(answers[i].studentAnswers[index]);
                    }
                }                
				return Ok(new { props , stuAns});
            }
            catch (Exception ex)
            {
                await _dingDing.SendBotMsg($"CoreAPI2，{_option.Location}，exam/find-summary-by-student()\n{ex.Message}", GroupNames.醍摩豆服務運維群組);
                return BadRequest();
            }

        }

        //查询学生活动列表
        [ProducesDefaultResponseType]
        //[AuthToken(Roles = "Student")]
        [HttpPost("find-all-by-student")]
        public async Task<IActionResult> findAllStudent(JsonElement requert)
        {
            //ResponseBuilder builder = ResponseBuilder.custom();
            //var (id, school) = HttpContext.GetAuthTokenInfo();
            try
            {
                if (!requert.TryGetProperty("id", out JsonElement id)) return BadRequest();
                //if (!requert.TryGetProperty("code", out JsonElement code)) return BadRequest();
                var client = _azureCosmos.GetCosmosClient();
                var query = $"select c.id,c.code,c.name,c.startTime,c.endTime,c.type,c.progress,c.school from c where array_contains(c.targetClassIds,'{id}') and c.progress = 'going'";
                List<object> props = new List<object>();
                await foreach (var item in client.GetContainer("TEAMModelOS", "Common").GetItemQueryStreamIterator(queryText: query))
                {
                    using var json = await JsonDocument.ParseAsync(item.ContentStream);
                    if (json.RootElement.TryGetProperty("_count", out JsonElement count) && count.GetUInt16() > 0)
                    {
                        foreach (var obj in json.RootElement.GetProperty("Documents").EnumerateArray())
                        {
                            props.Add(obj.ToObject<object>());
                        }
                    }
                }

                return Ok(props);
            }
            catch (Exception ex)
            {
                await _dingDing.SendBotMsg($"CoreAPI2，{_option.Location}，exam/find-all-by-student\n{ex.Message}", GroupNames.醍摩豆服務運維群組);
                return BadRequest();
            }

        }
        private  class StanderAnswers {
            List<List<string>> standard { get; set; }
            List<double> points { get; set; }
        }
        /// <summary>
        /// 查询单个作答信息  试卷id ,
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        /*[ProducesDefaultResponseType]
        //[AuthToken(Roles = "Teacher")]
        [HttpPost("find-by-idPk")]
        public async Task<BaseResponse> findByIdPk(IdPk request)
        {
            ResponseBuilder builder = ResponseBuilder.custom();
            ExamRecord record = await _azureCosmos.FindByIdPk<ExamRecord>(request.id,request.pk);
            if (record != null) {
                //处理客观题自动阅卷
                Paper paper = await _azureCosmos.FindByIdPk<Paper>(record.id, record.examCode);
                //允许自动对客观题阅卷及，及阅卷状态大于2 已完成.
                if (paper.markConfig != null && paper.markConfig.auto && record.mark<2) {
                    // TODO 需要处理已经打分的作答，以防止手动改分后被冲掉。

                    if (paper.answers.IsNotEmpty() && record.answers.IsNotEmpty()) {
                        autoMark(paper.item, paper.answers, record.answers, paper.markConfig);
                    }
                }
            }
            return builder.Data(record).build();
        }*/


        #region private        

        /*private List<Answer> autoMark(List<ItemInfo> items ,List<Answer> stdAnswers, List<Answer> stuAnswers ,MarkConfig markConfig) {
            int size = stuAnswers.Count;
            for (int i = 0; i < size; i++) {
                //如果当前题目已经打分则直接跳过。
                if (stuAnswers[i].score > 0) {
                    stuAnswers[i].mark = 1;
                    continue;
                }
                //客观题
                if (stuAnswers[i].type.Equals("single") || stuAnswers[i].type.Equals("multiple") || stuAnswers[i].type.Equals("judge")) {
                    stuAnswers[i].mark = 1;
                    //多选题单独处理
                    if (stuAnswers[i].type.Equals("multiple"))
                    {
                        List<string> stuAns = stuAnswers[i].ans;
                        Answer stdAnswer = stdAnswers.Where(x =>x.num== stuAnswers[i].num).FirstOrDefault();
                        if (stdAnswer != null) {
                            //处理多选答案是否有选错的 选错的则直接0分 ，如果少选则处理部分分数
                            bool right = true;
                            List<string> rightStr = new List<string>();
                            if (stuAns.IsNotEmpty())
                            {
                                foreach (string stuAn in stuAns)
                                {
                                    if (!stdAnswer.ans.Contains(stuAn))
                                    {
                                        right = false;
                                        break;
                                    }
                                    else {
                                        rightStr.Add(stuAn);
                                    }
                                }
                            }
                            else {
                                right = false;
                            }
                            if (right && rightStr.IsNotEmpty() && rightStr.Count != stdAnswer.ans.Count)
                            {

                                if (markConfig.type == 1)
                                {
                                    //1多选漏选不得分
                                    stuAnswers[i].score = 0;
                                }
                                else if(markConfig.type==3){
                                    stuAnswers[i].score=Math.Floor((double)(1.0 * rightStr.Count / stdAnswer.ans.Count * stdAnswer.score));
                                    if (stuAnswers[i].score == 0) {
                                        stuAnswers[i].score = 1;
                                    }
                                }
                                else if (markConfig.type == 4)
                                {
                                    stuAnswers[i].score = markConfig.score;
                                }
                                else
                                {   //2多选漏选得一半的分数（默认）
                                    stuAnswers[i].score = stdAnswer.score/2;

                                }
                            }
                            else {
                                //选错不得分
                                stuAnswers[i].score = 0;
                            }
                        }
                    }
                    else {
                        List<string> stuAns = stuAnswers[i].ans;
                        Answer stdAnswer = stdAnswers.Where(x => x.num == stuAnswers[i].num).FirstOrDefault();
                        if (stdAnswer != null&&stdAnswer.ans.IsNotEmpty()&& stuAns.IsNotEmpty()) {
                            if (stuAns[0].Equals(stdAnswer.ans[0]))
                            {
                                stuAnswers[i].score = stdAnswer.score;
                            }
                            else {
                                stuAnswers[i].score = 0;
                            }
                        }
                    }
                }
            }
            return stuAnswers;
        }*/

        #endregion
    }
}
