using Azure.Storage.Blobs.Models;
using Azure.Storage.Sas;
using HTEXLib;
using HTEXLib.Builders;
using HTEXLib.Helpers.ShapeHelpers;
using Microsoft.AspNetCore.Hosting;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Mvc;
using System;
using System.Collections.Generic;
using System.Globalization;
using System.IdentityModel.Tokens.Jwt;
using System.IO;
using System.Linq;
using System.Net.Http;
using System.Text;
using System.Text.Json;
using System.Threading.Tasks;
using TEAMModelOS.Models.CommonInfo;
using TEAMModelOS.Models.Dto;
using TEAMModelOS.Models.PowerPoint;
using TEAMModelOS.SDK;
using TEAMModelOS.SDK.Context.Constant;
using TEAMModelOS.SDK.Context.Constant.Common;
using TEAMModelOS.SDK.DI;
using TEAMModelOS.SDK.Extension;
using TEAMModelOS.SDK.Module.AzureBlob.Configuration;
using TEAMModelOS.SDK.Module.AzureBlob.Container;
using TEAMModelOS.Services;
using TEAMModelOS.Servicess.Exam;
using TEAMModelOS.Servicess.PowerPoint.Implement;

namespace TEAMModelOS.Controllers
{
    [Route("api/[controller]")]
    [ApiController]
    public class ImportExerciseController : BaseController
    {
        public HtexGenerator htexGenerator { get; set; }
        //  private readonly IHtexService htexService;
        private readonly AzureStorageFactory _azureStorage;
        private readonly IWebHostEnvironment _webHostEnvironment;
        private  List<LangConfig> langConfigs { get; set; }
        private readonly IHttpClientFactory _clientFactory;
        public ImportExerciseController(   AzureStorageFactory azureStorage, IWebHostEnvironment webHostEnvironment, HtexGenerator htexGenerator, IHttpClientFactory clientFactory)
        {
            _clientFactory = clientFactory;
            this.htexGenerator = htexGenerator;
            _webHostEnvironment = webHostEnvironment;
            _azureStorage = azureStorage;
            string path = _webHostEnvironment.ContentRootPath + "/JsonFile/Core/LangConfig.json";
            FileStream fs = new FileStream(path, System.IO.FileMode.Open, FileAccess.Read, FileShare.ReadWrite);
            StreamReader sr = new StreamReader(fs, System.Text.Encoding.UTF8);
            String line;
            StringBuilder builder = new StringBuilder();
            while ((line = sr.ReadLine()) != null)
            {
                builder.Append(line.ToString());
            }

            sr.Close();
            string text = builder.ToString();
            langConfigs = text.ToObject<List<LangConfig>>();
        }
        /// <summary>
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        [HttpPost("LoadDoc")]
        [RequestSizeLimit(102_400_000_00)] //最大10000m左右
        public async Task<BaseResponse> LoadDoc([FromForm] IFormFile file)
        {
            ResponseBuilder responseBuilder = new ResponseBuilder();

            Dictionary<string, object> model = await HtexService.LoadDoc(_azureStorage, file);
            return responseBuilder.Data(model).build();
        }


        /// <summary>
        /// {"url":"https://***.blob.core.cn/xxx/1.pptx"}
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        [HttpPost("analysis-pptx")]
        [RequestSizeLimit(102_400_000_00)] //最大10000m左右
        public async Task<IActionResult> AnalysisPPTX(JsonElement request)
        {
            string id_token = HttpContext.GetXAuth("IdToken");
            if (string.IsNullOrEmpty(id_token)) return BadRequest();
            var jwt = new JwtSecurityToken(id_token);
            if (!jwt.Payload.Iss.Equals("account.teammodel", StringComparison.OrdinalIgnoreCase)) return BadRequest();
            var id = jwt.Payload.Sub;

            request.TryGetProperty("code", out JsonElement code);
            string azureBlobSAS = System.Web.HttpUtility.UrlDecode(code.ToString(), Encoding.UTF8);
            (string, string) a = BlobUrlString(azureBlobSAS);
            string ContainerName = a.Item1;
            string BlobName = a.Item2;
            bool flg = IsBlobName(BlobName);
            var codes = code.ToString().Split("/");
            var FileName = codes[codes.Length - 1];
            if (flg)
            {
                //TODO 需驗證
                BlobAuth blobAuth = _azureStorage.GetBlobSasUriRead(ContainerName, BlobName);
                var response = await _clientFactory.CreateClient().GetAsync(new Uri(blobAuth.url + blobAuth.sas));
                response.EnsureSuccessStatusCode();
                Stream stream=  await response.Content.ReadAsStreamAsync();
                HTEXLib.Htex htex = await PPTXTranslator(id, FileName, stream);
                return Ok(JsonSerializer.Deserialize<JsonElement>(json: JsonHelper.ToJson(htex, ignoreNullValue: true)));
            }
            else { return BadRequest("不是正确的Blob链接！"); }
        }
        private static (string, string) BlobUrlString(string sasUrl)
        {
            sasUrl = sasUrl.Substring(8);
            string[] sasUrls = sasUrl.Split("/");
            string ContainerName;
            ContainerName = sasUrls[1].Clone().ToString();
            string item = sasUrls[0] + "/" + sasUrls[1] + "/";
            string blob = sasUrl.Replace(item, "");
            return (ContainerName, blob);
        }
        public static bool IsBlobName(string BlobName)
        {
            return System.Text.RegularExpressions.Regex.IsMatch(BlobName,
             @"(?!((^(con)$)|^(con)\\..*|(^(prn)$)|^(prn)\\..*|(^(aux)$)|^(aux)\\..*|(^(nul)$)|^(nul)\\..*|(^(com)[1-9]$)|^(com)[1-9]\\..*|(^(lpt)[1-9]$)|^(lpt)[1-9]\\..*)|^\\s+|.*\\s$)(^[^\\\\\\:\\<\\>\\*\\?\\\\\\""\\\\|]{1,255}$)");
        }
        /// <summary>
        /// 
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        [HttpPost("upload-pptx")]
        [RequestSizeLimit(102_400_000_00)] //最大10000m左右
        public async Task<IActionResult> UploadPPTX([FromForm] IFormFile file)
        {

            string id_token = HttpContext.GetXAuth("IdToken");
            if (string.IsNullOrEmpty(id_token)) return BadRequest();
            var jwt = new JwtSecurityToken(id_token);
            if (!jwt.Payload.Iss.Equals("account.teammodel", StringComparison.OrdinalIgnoreCase)) return BadRequest();
            var id = jwt.Payload.Sub;
            if (!FileType.GetExtention(file.FileName).ToLower().Equals("pptx"))
            {
                return BadRequest("type is not pptx！");
            }
            string FileName = file.FileName;
            Stream streamFile = file.OpenReadStream();

            HTEXLib.Htex htex = await PPTXTranslator(id, FileName, streamFile);
            return Ok(JsonSerializer.Deserialize<JsonElement>(json: JsonHelper.ToJson(htex, ignoreNullValue: true)));
        }

        private async Task<HTEXLib.Htex> PPTXTranslator(string id, string FileName, Stream streamFile)
        {
            var day = new DateTimeOffset(DateTime.UtcNow).ToString("yyyyMMdd", DateTimeFormatInfo.InvariantInfo);
            string shaCode = ShaHashHelper.GetSHA1(streamFile);
            HTEXLib.Htex htex = htexGenerator.Generator(streamFile);
            htex.name = FileName;
            var slides = htex.slides;
            int index = 1;
            List<Task> tasks = new List<Task>();
            foreach (var slide in slides)
            {
                string text = JsonHelper.ToJson(slide, ignoreNullValue: false);
                tasks.Add(_azureStorage.UploadFileByContainer(id, text, "pptx", day + "/" + shaCode + "/" + index + ".json", false)
                    .ContinueWith((Task<AzureBlobModel> blob) =>
                    {
                        htex.urls.Add(blob.Result.BlobUrl);
                    })
                    );
                // var blob=  await _azureStorage.UploadFileByContainer(id, text, "pptx", day+"/"+ shaCode +"/"+ index + ".json", false);
                // htex.urls.Add(blob.BlobUrl);
                index++;
            }
            await Task.WhenAll(tasks);
            htex.slides = null;
            Dictionary<string, Store> dict = new Dictionary<string, Store>();
            List<Task> tasksFiles = new List<Task>();
            foreach (var key in htex.stores.Keys)
            {
                if (key.EndsWith(".wdp") || key.EndsWith(".xlsx"))
                {
                    htex.stores.Remove(key);
                    continue;
                }
                var store = htex.stores[key];
                Store str = new Store() { path = key, contentType = store.contentType, isLazy = store.isLazy };
                if (!store.isLazy && store.contentType != null && ContentTypeDict.extdict.TryGetValue(store.contentType, out string ext) && store.url.Contains(";base64,"))
                {
                    string[] strs = store.url.Split(',');
                    Stream stream = new MemoryStream(Convert.FromBase64String(strs[1]));
                    var urlstrs = key.Split("/");
                    var name = urlstrs[urlstrs.Length - 1];
                    tasksFiles.Add(_azureStorage.UploadFileByContainer(id, stream, "pptx", day + "/" + shaCode + "/" + name, false)
                        .ContinueWith((Task<AzureBlobModel> blob) =>
                        {
                            str.url = blob.Result.BlobUrl;
                        })
                        );
                    // var blob = await _azureStorage.UploadFileByContainer(id, stream, "pptx", day + "/" + shaCode + "/" + name, false);
                    //  str.url = blob.BlobUrl;

                }
                else
                {
                    str.url = store.url;
                }
                dict.TryAdd(key, str);
            }
            await Task.WhenAll(tasksFiles);
            htex.stores = dict;
            return htex;
        }


        /// <summary>
        /// docUrl
        /// folder
        /// shaCode
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        [HttpPost("UploadWord")]
        [RequestSizeLimit(102_400_000_00)] //最大10000m左右
        public async Task<IActionResult> UploadWord([FromForm] IFormFile file)
        {
          //  ResponseBuilder responseBuilder = new ResponseBuilder();
            if (!FileType.GetExtention(file.FileName).ToLower().Equals("docx"))
            {
                return BadRequest(new Dictionary<string, object> { {"msg", "type is not docx！" },{ "code",ResponseCode.FAILED} });
            }
            Dictionary<string, object> model = await ImportExerciseService.UploadWord(_azureStorage, file);
            return Ok(model);
        }

        /// <summary>
        /// htmlString 
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        [HttpPost("AnalyzeHtml")]
        public  IActionResult  AnalyzeHtml(JsonElement request)
        {
            //ResponseBuilder builder = ResponseBuilder.custom();
            Dictionary<string, object> dict = new Dictionary<string, object>();
            var emobj = request.EnumerateObject();
            while (emobj.MoveNext())
            {
                dict[emobj.Current.Name] = emobj.Current.Value;
            }
            bool flag = dict.TryGetValue("htmlString", out object htmlString);
            bool flagLang = dict.TryGetValue("lang", out object lang);
            if (flag && htmlString != null && !string.IsNullOrEmpty(htmlString.ToString()))
            {
                LangConfig langConfig= langConfigs.Where(x => x.Lang == lang.ToString()).FirstOrDefault();
                HtmlAnalyzeService htmlAnalyzeService = new HtmlAnalyzeService(langConfig);
                List<ItemInfo> exercises = htmlAnalyzeService.AnalyzeWordAsync(htmlString.ToString());
                return Ok(exercises);
            }
            else
            {
                return BadRequest();
            }
        }
        /// <summary>
        /// htmlString 
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        [HttpPost("HtmlToHtex")]
        public async Task<BaseResponse> HtmlToHtex(JsonElement request)
        {
            ResponseBuilder builder = ResponseBuilder.custom();
/*            Dictionary<string, object> dict = new Dictionary<string, object>();
            var emobj = request.EnumerateObject();
            while (emobj.MoveNext())
            {
                dict[emobj.Current.Name] = emobj.Current.Value;
            }*/
            bool flag = request.TryGetProperty("htmlString", out JsonElement htmlString);
            bool flagLang = request.TryGetProperty("lang", out JsonElement lang);
            if (flag && htmlString.ToString() != null && !string.IsNullOrEmpty(htmlString.ToString()))
            {
                LangConfig langConfig = langConfigs.Where(x => x.Lang == lang.ToString()).FirstOrDefault();
                HtmlAnalyzeService htmlAnalyzeService = new HtmlAnalyzeService(langConfig);
                Models.PowerPoint.Htex exercises = await HtexService.AnalyzeHtmlToHtex(_azureStorage, htmlString.ToString(), lang.ToString(), htmlAnalyzeService);
                return builder.Data(exercises).build();
            }
            else
            {
                return builder.Data(null).build();
            }
        }
    }
}
