using Microsoft.AspNetCore.Mvc;
using System;
using System.Collections.Generic;
using System.Text;
using System.Threading.Tasks;
using TEAMModelOS.Models;
using TEAMModelOS.SDK;
using TEAMModelOS.SDK.DI;
using TEAMModelOS.SDK.Helper.Common.CollectionHelper;
using TEAMModelOS.SDK.Context.Exception;
using System.Text.Json;
using TEAMModelOS.Models.SchoolInfo;
using TEAMModelOS.SDK.Extension;
using Azure.Cosmos;
using Microsoft.AspNetCore.Http;
using Microsoft.Extensions.Options;
using System.IO;
using System.Dynamic;
using TEAMModelOS.SDK.Context.Configuration;
using System.Net.Http;
using System.Net;
using Newtonsoft.Json;
using System.Linq;

namespace TEAMModelOS.Controllers
{
    [ProducesResponseType(StatusCodes.Status200OK)]
    [ProducesResponseType(StatusCodes.Status400BadRequest)]
    //[Authorize(Roles = "IES5")]
    [Route("school/init")]
    [ApiController]
    public class SchoolController : ControllerBase
    {

        public AzureCosmosFactory _azureCosmos;
        private readonly DingDing _dingDing;
        private readonly Option _option;
        public SchoolController(AzureCosmosFactory azureCosmos, DingDing dingDing, IOptionsSnapshot<Option> option)
        {
            _azureCosmos = azureCosmos;
            _dingDing = dingDing;
            _option = option?.Value; 
        }
        /// <summary>
        /// 保存或更新学校
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        [ProducesDefaultResponseType]
        //[AuthToken(Roles = "admin")]
        [HttpPost("upsert")]
        public async Task<IActionResult> Upsert(School requert)
        {
            var (_, _, _, school) = HttpContext.GetAuthTokenInfo();
            try
            {
                School schoolInfo = new School();
                var client = _azureCosmos.GetCosmosClient();
                requert.ttl = -1;
                requert.pk = typeof(School).Name;
                var response = await client.GetContainer("TEAMModelOS", "School").ReadItemStreamAsync(requert.id, new PartitionKey($"Base"));
                if (response.Status == 200)
                {
                     
                     schoolInfo = await _azureCosmos.GetCosmosClient().GetContainer("TEAMModelOS", "School").UpsertItemAsync(requert, new PartitionKey($"Base"));
                }
                else
                {
                      requert.code = "Base";
                      schoolInfo = await _azureCosmos.GetCosmosClient().GetContainer("TEAMModelOS", "School").CreateItemAsync(requert, new PartitionKey($"Base"));
                    
                }
                return Ok(new  { schoolInfo });
            }
            catch (Exception ex)
            {
                await _dingDing.SendBotMsg($"CoreAPI2，{_option.Location}，school/upsert()\n{ex.Message}", GroupNames.醍摩豆服務運維群組);
                return BadRequest();
            }
            /*ResponseBuilder builder = ResponseBuilder.custom();
            if (string.IsNullOrEmpty(request.id))
            {
                List<School> schools = await _azureCosmos.FindByDict<School>(new Dictionary<string, object> { { "id", request.schoolCode } });
                if (schools.IsNotEmpty())
                {
                    //return builder.Error(ResponseCode.DATA_EXIST, "学校已存在！").build();
                    return Ok(new { V = "学校已经存在！" });
                }
				request.code = "Base";
				//request.id = request.schoolCode;
			}
            else {
                *//*if (request.id.Equals(request.schoolCode))
                {*//*
                    School datas = await _azureCosmos.SaveOrUpdate<School>(request);
                    return Ok(new{ datas});
                *//*}
                else
                {
                    //return builder.Error(ResponseCode.PARAMS_ERROR, "id,schoolCode必须相同！").build();
                    return Ok(new { V = "id,schoolCode必须相同！" });
                }*/
            /*List<School> schools = await _azureCosmos.FindByDict<School>(new Dictionary<string, object> { { "id", request.id } });
            if (schools.IsEmpty())
            {
                return builder.Error(ResponseCode.PARAMS_ERROR, "id不存在，不能更新").build();
            }*//*
        }
        return Ok();*/
        }
        /// <summary>
        /// 查找学校
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        [ProducesDefaultResponseType]
        //[AuthToken(Roles = "teacher")]
        [HttpPost("find")]
        public async Task<IActionResult> Find(JsonElement requert)
        {
            if (!requert.TryGetProperty("school_code", out JsonElement school_code)) return BadRequest();
            var client = _azureCosmos.GetCosmosClient();
            List<School> schools = new List<School>();
            var query = $"select c.id,c.pk,c.code, c.name,c.region,c.province,c.city,c.timeZone,c.picture,c.size,c.period,c.campuses from c where c.id ='{school_code}'";
            await foreach (var item in client.GetContainer("TEAMModelOS", "School").GetItemQueryStreamIterator(queryText: query, requestOptions: new QueryRequestOptions() { PartitionKey = new PartitionKey("Base") }))
            {
                using var json = await JsonDocument.ParseAsync(item.ContentStream);

                if (json.RootElement.TryGetProperty("_count", out JsonElement count) && count.GetUInt16() > 0)
                {
                    foreach (var obj in json.RootElement.GetProperty("Documents").EnumerateArray())
                    {
                        schools.Add(obj.ToObject<School>());
                    }
                }
            }

            return Ok(new { schools });
            /* ResponseBuilder builder = ResponseBuilder.custom();

             if (request.TryGetProperty("code", out JsonElement code) &&! string.IsNullOrEmpty(code.ToString()))
             {
                 List<School> sc = await _azureCosmos.FindByDict<School>(request);
                 return builder.Data(sc).build();
             }
             else {
                 return builder.Data(null).build();
             }*/
        }
        /// <summary>
        /// 取得所有學校(只取基本資料)
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        [ProducesDefaultResponseType]
        [HttpPost("get-all-school-base")]
        public async Task<IActionResult> GetAllSchoolBaesInfo(JsonElement request)
        {
            var client = _azureCosmos.GetCosmosClient();
            List<object> schools = new List<object>();
            await foreach (var item in client.GetContainer("TEAMModelOS", "School").GetItemQueryStreamIterator(queryText: $"SELECT c.id, c.code, c.name, c.region, c.province, c.city, c.address, c.picture FROM c ", requestOptions: new QueryRequestOptions() { PartitionKey = new PartitionKey($"Base") }))
            {
                using var json = await JsonDocument.ParseAsync(item.ContentStream);
                if (json.RootElement.TryGetProperty("_count", out JsonElement count) && count.GetUInt16() > 0)
                {
                    foreach (var obj in json.RootElement.GetProperty("Documents").EnumerateArray())
                    {
                        schools.Add(obj.ToObject<object>());
                    }
                }
            }
            return Ok(new { schools });
        }

        /// <summary>
        /// 取得某學校基本資料 
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        [ProducesDefaultResponseType]
        [HttpPost("get-school-base")]
        public async Task<IActionResult> GetSchoolBaesInfo(JsonElement request)
        {
            if (!request.TryGetProperty("school_code", out JsonElement school_code)) return BadRequest();

            var client = _azureCosmos.GetCosmosClient();
            //取得學校學制、年級、教室
            List<object> classes = new List<object>();
            await foreach (var item in client.GetContainer("TEAMModelOS", "School").GetItemQueryStreamIterator(queryText: $"SELECT c.id, c.name, c.gradeId FROM c", requestOptions: new QueryRequestOptions() { PartitionKey = new PartitionKey($"Class-{school_code}") }))
            {
                var jsonc = await JsonDocument.ParseAsync(item.ContentStream);
                foreach (var classeinfo in jsonc.RootElement.GetProperty("Documents").EnumerateArray())
                {
                    dynamic classExtobj = new ExpandoObject();
                    classExtobj.id = classeinfo.GetProperty("id");
                    classExtobj.name = classeinfo.GetProperty("name");
                    classExtobj.gradeId = classeinfo.GetProperty("gradeId");
                    classes.Add(classExtobj);
                }
            }
            List<object> periods = new List<object>();
            List<object> grades = new List<object>();
            var responsesch = await client.GetContainer("TEAMModelOS", "School").ReadItemStreamAsync(school_code.ToString(), new PartitionKey($"Base"));
            if (responsesch.Status == 200)
            {
                var jsons = await JsonDocument.ParseAsync(responsesch.ContentStream);
                if (jsons.RootElement.TryGetProperty("period", out JsonElement periodJobj))
                {
                    foreach (var periodinfo in periodJobj.EnumerateArray())
                    {
                        dynamic periodExtobj = new ExpandoObject();
                        periodExtobj.id = periodinfo.GetProperty("id");
                        periodExtobj.name = periodinfo.GetProperty("name");
                        periods.Add(periodExtobj);
                        if (periodinfo.TryGetProperty("grades", out JsonElement gradesJobj))
                        {
                            foreach (var gradeinfo in gradesJobj.EnumerateArray())
                            {
                                dynamic gradeExtobj = new ExpandoObject();
                                gradeExtobj.id = gradeinfo.GetProperty("id");
                                gradeExtobj.name = gradeinfo.GetProperty("name");
                                gradeExtobj.periodId = periodinfo.GetProperty("id");
                                grades.Add(gradeExtobj);
                            }
                        }
                    }
                }
            }
            return Ok(new { periods, grades, classes });
        }

        /// <summary>
        /// 取得某學校產品資料 
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        [ProducesDefaultResponseType]
        [HttpPost("get-school-product")]
        public async Task<IActionResult> GetSchoolProductInfo(JsonElement request)
        {
            if (!request.TryGetProperty("school_code", out JsonElement school_code)) return BadRequest();

            var client = _azureCosmos.GetCosmosClient();
            List<SerialInfo> serial = new List<SerialInfo>();
            List<object> service = new List<object>();
            List<object> hard = new List<object>();
            List<object> uuidList = new List<object>(); //要向CoreService詢問deviceID及硬體資訊的UUID列表
            var response = await client.GetContainer("TEAMModelOSTemp", "School").ReadItemStreamAsync(school_code.ToString(), new PartitionKey("Product"));
            if (response.Status == 200)
            {
                var json = await JsonDocument.ParseAsync(response.ContentStream);
                if (json.RootElement.TryGetProperty("serial", out JsonElement serialJobj))
                {
                    foreach (var serialInfo in serialJobj.EnumerateArray())
                    {
                        serial.Add(serialInfo.ToObject<SerialInfo>());
                        serialInfo.TryGetProperty("deviceBound", out JsonElement deviceBoundJobj);
                        foreach (var deviceBoundRow in deviceBoundJobj.EnumerateArray())
                        {
                            dynamic uuidExtobj = new ExpandoObject();
                            uuidExtobj.serial = serialInfo.GetProperty("serial");
                            uuidExtobj.uuid = deviceBoundRow.GetProperty("uuid");
                            uuidExtobj.uuid2 = deviceBoundRow.GetProperty("uuid2");
                            uuidExtobj.deviceId = deviceBoundRow.GetProperty("deviceId");
                            uuidList.Add(uuidExtobj);
                        }
                    }
                }

                if (json.RootElement.TryGetProperty("service", out JsonElement serviceJobj))
                {
                    service.Add(serviceJobj.ToObject<object>());
                }

                if (json.RootElement.TryGetProperty("hard", out JsonElement hardJobj))
                {
                    hard.Add(hardJobj.ToObject<object>());
                }

                //取得DeviceInfo From Core [先做假的，之後再對接]
                List<object> serialResult = new List<object>();
                List<CoreUuid> coreUuidList = (List<CoreUuid>)GetDeviceFromCore(uuidList);
                foreach (SerialInfo serialRow in serial)
                {
                    var deviceBoundArray = new List<deviceBoundExt>();
                    List<CoreUuid> coreUuid = coreUuidList
                           .Where((CoreUuid x) => x.serial == serialRow.serial)
                           .ToList();
                    foreach (CoreUuid deviceRow in coreUuid)
                    {
                        deviceBoundExt deviceBoundRow = new deviceBoundExt();
                        deviceBoundRow.uuid = deviceRow.uuid;
                        deviceBoundRow.uuid2 = deviceRow.uuid2;
                        deviceBoundRow.os = deviceRow.os;
                        deviceBoundRow.deviceId = deviceRow.deviceId;
                        deviceBoundRow.ip = deviceRow.ip;
                        deviceBoundRow.cpu = deviceRow.cpu;
                        deviceBoundRow.pcname = deviceRow.pcname;
                        deviceBoundRow.ram = deviceRow.ram;
                        deviceBoundArray.Add(deviceBoundRow);
                    }
                    serialRow.deviceBound = deviceBoundArray;
                }


            }



            return Ok(new { serial, service, hard });
        }

        /// <summary>
        /// 取得CoreDevice資訊
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        private List<CoreUuid> GetDeviceFromCore(List<object> uuidList)
        {
            List<CoreUuid> list = new List<CoreUuid>();
            try
            {
                //string url = BaseConfigModel.Configuration["HaBookAuth:CoreId:userinfo"];
                //HttpClient client = new HttpClient();
                //var content = new StringContent(JsonConvert.SerializeObject(uuidList), Encoding.UTF8, "application/json");
                //HttpResponseMessage responseMessage = await client.PostAsync(url, content);
                //if (responseMessage.StatusCode == HttpStatusCode.OK)
                //{
                //    string responseBody = responseMessage.Content.ReadAsStringAsync().Result;
                //    return Ok(responseBody);
                //}
                //else
                //{
                //    return BadRequest();
                //}

                CoreUuid CoreUuid1 = new CoreUuid();
                CoreUuid1.serial = "3222IAVN-H4RS-VF5L-YSCJ-ZXYW";
                CoreUuid1.uuid = "3C970E5CD649";
                CoreUuid1.uuid2 = "E006E6CE1D28";
                CoreUuid1.deviceId = "2d776529-ac7b-4197-b981-3ae3370fe686";
                CoreUuid1.ip = "171.216.152.131";
                CoreUuid1.pcname = "ONLY-PC";
                CoreUuid1.os = "Microsoft Windows NT 6.1.7601 Service Pack 1";
                CoreUuid1.cpu = "Intel(R) Core(TM) i5-3210M CPU @ 2.50GHz";
                CoreUuid1.ram = 3287;
                list.Add(CoreUuid1);
                CoreUuid CoreUuid2 = new CoreUuid();
                CoreUuid2.serial = "3222IAVN-8MKL-9N6F-C14T-QK39";
                CoreUuid2.uuid = "000C2909298F";
                CoreUuid2.uuid2 = null;
                CoreUuid2.deviceId = "a5b21a8d-b204-40f4-b28b-c7df34cda0a9";
                CoreUuid2.ip = "182.148.142.235";
                CoreUuid2.pcname = null;
                CoreUuid2.os = "Microsoft Windows NT 6.1.7601 Service Pack 1";
                CoreUuid2.cpu = "Intel(R) Core(TM) i7-8700K CPU @ 3.70GHz";
                CoreUuid2.ram = 6143;
                list.Add(CoreUuid2);
                CoreUuid CoreUuid3 = new CoreUuid();
                CoreUuid3.serial = "12DLT43F-BTXK-W65T-9P6J-DVAR";
                CoreUuid3.uuid = null;
                CoreUuid3.uuid2 = "A483E742B0FF";
                CoreUuid3.deviceId = "efb9adc2-0713-401a-8f17-46129cf3b849";
                CoreUuid3.ip = "110.185.29.13";
                CoreUuid3.pcname = "DESKTOP-D4OMD85";
                CoreUuid3.os = "Microsoft Windows NT 10.0.18362.0";
                CoreUuid3.cpu = "Intel(R) Core(TM) i5-8257U CPU @ 1.40GHz";
                CoreUuid3.ram = 8036;
                list.Add(CoreUuid3);
                CoreUuid CoreUuid4 = new CoreUuid();
                CoreUuid4.serial = "12DLT43F-TOGN-OGVR-MYVR-6PSA";
                CoreUuid4.uuid = "94C691231A5F";
                CoreUuid4.uuid2 = null;
                CoreUuid4.deviceId = "a4c0bf19-db01-4f3c-8d1e-f3074476349f";
                CoreUuid4.ip = "118.114.15.247";
                CoreUuid4.pcname = "PC-201909100955";
                CoreUuid4.os = "Microsoft Windows NT 10.0.14393.0";
                CoreUuid4.cpu = "Intel(R) Core(TM) i5-7400 CPU @ 3.00GHz";
                CoreUuid4.ram = 16291;
                list.Add(CoreUuid4);
                CoreUuid CoreUuid5 = new CoreUuid();
                CoreUuid5.serial = "12DLT43F-TOGN-OGVR-MYVR-6PSA";
                CoreUuid5.uuid = "D8CB8A780144";
                CoreUuid5.uuid2 = null;
                CoreUuid5.deviceId = "699c8847-f9cd-4f8d-9799-2f5555e46e34";
                CoreUuid5.ip = "183.220.92.210";
                CoreUuid5.pcname = "USER-20180716GZ";
                CoreUuid5.os = "Microsoft Windows NT 10.0.17134.0";
                CoreUuid5.cpu = "Intel(R) Core(TM) i5-4590 CPU @ 3.30GHz";
                CoreUuid5.ram = 16287;
                list.Add(CoreUuid5);
                CoreUuid CoreUuid6 = new CoreUuid();
                CoreUuid6.serial = "12DLT43F-TOGN-OGVR-MYVR-6PSA";
                CoreUuid6.uuid = null;
                CoreUuid6.uuid2 = "C49DED0BB2A9";
                CoreUuid6.deviceId = "44af0c54-7927-4a26-8503-9dcb62272e95";
                CoreUuid6.ip = "118.114.15.247";
                CoreUuid6.pcname = "DESKTOP-S4JOQCB";
                CoreUuid6.os = "Microsoft Windows NT 10.0.18362.0";
                CoreUuid6.cpu = "Intel(R) Core(TM) i5-7300U CPU @ 2.60GHz";
                CoreUuid6.ram = 4021;
                list.Add(CoreUuid6);

                return list;
            }
            catch (Exception ex)
            {
                return list;
            }
        }
    }
}
