using Microsoft.AspNetCore.Mvc;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using TEAMModelOS.SDK.Context.Exception;
using TEAMModelOS.SDK;
using TEAMModelOS.SDK.Helper.Common.CollectionHelper;
using TEAMModelOS.SDK.DI;

using TEAMModelOS.Models;
using System.Text.Json;
using TEAMModelOS.SDK.Helper.Common.StringHelper;
using TEAMModelOS.Models.CommonInfo;
using Microsoft.AspNetCore.Http;
using Azure.Cosmos;
using TEAMModelOS.SDK.Extension;

namespace TEAMModelOS.Controllers
{

    [ProducesResponseType(StatusCodes.Status200OK)]
    [ProducesResponseType(StatusCodes.Status400BadRequest)]
    //[Authorize(Roles = "IES5")]
    [Route("common/resource")]
    [ApiController]
    public class ResourceController : ControllerBase
    {
        private readonly AzureStorageFactory _azureStorage;
        private readonly AzureCosmosFactory _azureCosmos;
        private readonly SnowflakeId _snowflakeId;
        public ResourceController(AzureStorageFactory azureStorage, AzureCosmosFactory azureCosmos, SnowflakeId snowflakeId)
        {
            _azureStorage = azureStorage;
            _azureCosmos = azureCosmos;
            _snowflakeId = snowflakeId;
        }

        /// <summary>
        /// 保存或更新SyllabusResource
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        [ProducesDefaultResponseType]
        //[AuthToken(Roles = "Teacher")]
        [HttpPost("upsertAll")]
        public async Task<IActionResult> UpsertAll(List<Resource> request)
        {
            //ResponseBuilder builder = ResponseBuilder.custom();
            if (request.IsNotEmpty())// != null )
            {
                List<Resource> resource = request;
                foreach (Resource item in resource)
                {
                    //item.code = 
                    if (item.id == null)
                    {
                        item.id = _snowflakeId.NextId()+""; //item.sha1Code + "-" + item.extension + "-" + item.size;
                        
                    }
                }
                await _azureCosmos.SaveOrUpdateAll(resource);
                //builder.Data("保存或新增成功");
            }
            return Ok("保存或新增成功");
        }


        /// <summary>
        /// 获取SyllabusResource
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        [ProducesDefaultResponseType]
        //[AuthToken(Roles = "Teacher")]
        [HttpPost("find")]
        public async Task<IActionResult> Find(JsonElement requert)
        {

            var client = _azureCosmos.GetCosmosClient();
            if (!requert.TryGetProperty("school_code", out JsonElement school_code)) return BadRequest();

            List<object> Resources = new List<object>();
            var query = $"select c.id,c.fileName,c.extension, c.contentType,c.type,c.blobUrl,c.createTime,c.range from c order by c.createTime desc";
            await foreach (var item in client.GetContainer("TEAMModelOS", "Common").GetItemQueryStreamIterator(queryText: query, requestOptions: new QueryRequestOptions() { PartitionKey = new PartitionKey($"Resource-{school_code}") }))
            {
                using var json = await JsonDocument.ParseAsync(item.ContentStream);

                if (json.RootElement.TryGetProperty("_count", out JsonElement count) && count.GetUInt16() > 0)
                {
                    foreach (var obj in json.RootElement.GetProperty("Documents").EnumerateArray())
                    {
                        Resources.Add(obj.ToObject<object>());
                    }
                }
            }

            return Ok(new { Resources });

            // request.@params.TryAdd("PartitionKey", request.lang);
            /*ResponseBuilder builder = ResponseBuilder.custom();
            if (StringHelper.getKeyCount(request) > 0)
            {
                List<Resource> data = new List<Resource>();
                data = await _azureCosmos.FindByDict<Resource>(request);

                return builder.Data(data.OrderBy(m => m.createTime)).Extend(new Dictionary<string, object> { { "count", data.IsNotEmpty() ? data.Count : 0 } }).build();
            }
            else {

                return builder.Error(ResponseCode.PARAMS_ERROR, "参数异常！").build();
            }*/

        }


        /// <summary>
        /// 获取SyllabusResource
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        [ProducesDefaultResponseType]
        //[AuthToken(Roles = "Teacher")]
        [HttpPost("find-by-id")]
        public async Task<IActionResult> FindSyllabusResourceById(JsonElement requert)
        {
            var client = _azureCosmos.GetCosmosClient();
            if (!requert.TryGetProperty("school_code", out JsonElement school_code)) return BadRequest();
            if (!requert.TryGetProperty("ids", out JsonElement id)) return BadRequest();
            string info = "";
            for (int i = 0; i < id.GetArrayLength(); i++)
            {
                info += id[i].ToJsonString() + ",";
            }
            List<object> Resources = new List<object>();
            var query = $"select c.id,c.fileName,c.extension, c.contentType,c.type,c.blobUrl,c.createTime,c.range from c  where c.id in ({info[0..^1]})";
            await foreach (var item in client.GetContainer("TEAMModelOS", "Common").GetItemQueryStreamIterator(queryText: query, requestOptions: new QueryRequestOptions() { PartitionKey = new PartitionKey($"Resource-{school_code}") }))
            {
                using var json = await JsonDocument.ParseAsync(item.ContentStream);

                if (json.RootElement.TryGetProperty("_count", out JsonElement count) && count.GetUInt16() > 0)
                {
                    foreach (var obj in json.RootElement.GetProperty("Documents").EnumerateArray())
                    {
                        Resources.Add(obj.ToObject<object>());
                    }
                }
            }

            return Ok(new { Resources });

            // request.@params.TryAdd("PartitionKey", request.lang);
            /* ResponseBuilder builder = ResponseBuilder.custom();
             if (request != null)
             {
                 List<Resource> data = new List<Resource>();
                 data = await _azureCosmos.FindByIds<Resource>(request);

                 return builder.Data(data.OrderBy(m => m.createTime)).Extend(new Dictionary<string, object> { { "count", data.IsNotEmpty() ? data.Count : 0 } }).build();



             }
             else
             { return builder.Error(ResponseCode.PARAMS_ERROR, "参数异常！").build(); }*/
        }


        /// <summary>
        /// 删除 SyllabusResource
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        [ProducesDefaultResponseType]
        //[AuthToken(Roles = "Teacher")]
        [HttpPost("delete")]
        public async Task<IActionResult> DeleteSyllabusResource(JsonElement request)
        {
            ResponseBuilder builder = ResponseBuilder.custom();
            List<Resource> syllabusResources =  await Delete<Resource>(request, builder);
            foreach (Resource syllabusResource  in syllabusResources) {

                string blobUrl = syllabusResource.blobUrl;
                if (!string.IsNullOrEmpty(blobUrl))
                {
                  //  await _azureBlobDBRepository.Deleteblob(blobUrl);
                }
                string compressUrl = syllabusResource.compressUrl;
                if (!string.IsNullOrEmpty(compressUrl))
                {
                   // await _azureBlobDBRepository.Deleteblob(compressUrl);
                }
            }

            return Ok();
        }

        private async Task<List<T>> Delete<T>(JsonElement request, ResponseBuilder builder)where T:ID
        {
            //Dictionary<string, object> dict = new Dictionary<string, object>();
            var emobj = request.EnumerateObject();
            int keys = 0;
            while (emobj.MoveNext())
            {
                keys++;
               //dict[emobj.Current.Name] = emobj.Current.Value;
            }
            if (keys > 0)
            {
                List<T> schoolBlocks = await _azureCosmos.FindByDict<T>(request);
                Type t = typeof(T);
                if (schoolBlocks.IsNotEmpty())
                {
                    await _azureCosmos.DeleteAll<T>(schoolBlocks);
                    builder.Data("删除成功");
                }

                return schoolBlocks;
            }
            else {
                  builder.Error(ResponseCode.PARAMS_ERROR, "参数异常！");
                return null;
            }

        }

        /// <summary>
        /// 查询 资源被引用次数
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        [ProducesDefaultResponseType]
        //[AuthToken(Roles = "Teacher")]
        [HttpPost("reference")]
        public async Task<IActionResult> Reference(JsonElement request)
        {
            ResponseBuilder builder = ResponseBuilder.custom();
            Dictionary<string, object> dict = new Dictionary<string, object>();
            var emobj = request.EnumerateObject();
            while (emobj.MoveNext())
            {
                dict[emobj.Current.Name] = emobj.Current.Value;
            }
            List<ResourceReference> data = new List<ResourceReference>();
            data = await _azureStorage.FindListByDict<ResourceReference>(dict);
            //return builder.Data(data.OrderBy(m => m.CreateTime)).Extend(new Dictionary<string, object> { { "count", data.Count } }).build();
            return Ok(new { OrderedEnumerables = data.OrderBy(m => m.CreateTime), data.Count });
        }
    }
}