using Microsoft.AspNetCore.Mvc;
using System.Collections.Generic;
using System.Threading.Tasks;
using TEAMModelOS.SDK.Context.Exception;
using TEAMModelOS.SDK;

using TEAMModelOS.SDK.DI;
using TEAMModelOS.SDK.Helper.Common.CollectionHelper;
using System;
using TEAMModelOS.SDK.Helper.Common.JsonHelper;
using System.Linq;
using TEAMModelOS.Models;
using System.Text.Json;
using TEAMModelOS.SDK.Extension;
using TEAMModelOS.Models.CommonInfo;
using Microsoft.AspNetCore.Http;

namespace TEAMModelOS.Controllers
{
    [ProducesResponseType(StatusCodes.Status200OK)]
    [ProducesResponseType(StatusCodes.Status400BadRequest)]
    //[Authorize(Roles = "IES5")]
    [Route("common/syllabus")]
    [ApiController]
    //[Authorize]
    public class SyllabusController : ControllerBase
    {
        private readonly AzureCosmosFactory _azureCosmos;


        public SyllabusController(AzureCosmosFactory azureCosmos)
        {
            _azureCosmos = azureCosmos;
        }


        /// <summary>
        /// 批量保存或更新课纲
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        [ProducesDefaultResponseType]
        [HttpPost("upsert-tree")]
        public async Task<IActionResult> SaveOrUpdateAsTree(List<SyllabusTree> request)
        {
            //ResponseBuilder builder = ResponseBuilder.custom();
            List<SyllabusTree> treess = await SaveOrUpdateToTree(request);
            //return builder.Data(treess).build();
            return Ok(treess);
        }


        /// <summary>
        /// 按节点新增课纲
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        [ProducesDefaultResponseType]
        [HttpPost("upsert-nodes")]
        public async Task<IActionResult> SaveOrUpdateAsNodes(List<SyllabusNode> request)
        {
            ResponseBuilder builder = ResponseBuilder.custom();
            List<Syllabus> syllabuses = await SaveOrUpdateToNodes(request);
            if (syllabuses.IsNotEmpty()) return Ok(request);
            else return Ok(syllabuses);
        }

        /// <summary>
        /// 删除课纲
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        [ProducesDefaultResponseType]
        [HttpPost("delete")]
        public async Task<IActionResult> Delete(JsonElement request)
        {
            ResponseBuilder builder = ResponseBuilder.custom();
            Dictionary<string, object> dict = new Dictionary<string, object>();
            var emobj = request.EnumerateObject();
            while (emobj.MoveNext())
            {
                dict[emobj.Current.Name] = emobj.Current.Value;
            }
            if (dict.Keys.Count > 0)
            {
                List<Syllabus> syllabuses = await DeleteSyllabus(dict);

                if (syllabuses.IsNotEmpty())
                {
                    return Ok(syllabuses);
                }
                else
                {
                    return Ok("全部删除成功");

                }
            }
            else
            {
                //return builder.Error(ResponseCode.PARAMS_ERROR, "参数异常！").build();
                return Ok(new { ResponseCode.PARAMS_ERROR, V = "参数异常！" });
            }


        }

        /// <summary>
        /// 查找课纲 
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        [ProducesDefaultResponseType]
        [HttpPost("find")]
        public async Task<IActionResult> Find(JsonElement request)
        {
            ResponseBuilder builder = ResponseBuilder.custom();
            Dictionary<string, object> dict = new Dictionary<string, object>();
            var emobj = request.EnumerateObject();
            while (emobj.MoveNext())
            {
                dict[emobj.Current.Name] = emobj.Current.Value;
            }
            List<SyllabusTree> treess = await Find(dict);
            //return builder.Data(treess).build();
            return Ok(treess);
        }


        #region private       

        /// <summary>
        /// 查找课纲 
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        private async Task<List<SyllabusTree>> Find(Dictionary<string, object> request)
        {
            Dictionary<string, object> dict = new Dictionary<string, object>();
            foreach (KeyValuePair<string, object> keyValuePair in request)
            {
                dict.Add("children[*]." + keyValuePair.Key, keyValuePair.Value);
            }
            List<Syllabus> data = await _azureCosmos.FindByDict<Syllabus>(dict);
            List<SyllabusTree> treess = new List<SyllabusTree>();
            if (data.IsNotEmpty())
            {
                Dictionary<string, Syllabus> syllabuses = new Dictionary<string, Syllabus>();
                data.ForEach(x => syllabuses.TryAdd(x.id, x));
                data = new List<Syllabus>(syllabuses.Values);

                foreach (Syllabus item in data)
                {

                    List<SyllabusNode> nodedata = item.children;
                    if (request.TryGetValue("id", out object id))
                    {
                        foreach (SyllabusNode syllabus in nodedata)
                        {
                            if (syllabus.id == id.ToString())
                            {
                                List<SyllabusNode> nodes = new List<SyllabusNode>();
                                nodes.Add(syllabus);
                                await FindByPid(syllabus, nodes, nodedata);
                                nodedata = nodes;
                                break;
                            }
                        }
                    }
                    List<SyllabusTree> treess1 = ListToTree(nodedata);
                    treess.AddRange(treess1);

                    //treess.AddRange(treess1);
                }

                // return builder.Data(treess).build();
            }
            // else return builder.Data(data).build();
            return treess;

        }

        private async Task<List<SyllabusNode>> FindByPid(SyllabusNode data, List<SyllabusNode> nodes, List<SyllabusNode> nodedata)
        {
            foreach (SyllabusNode syllabus in nodedata)
            {
                if (syllabus.pid == data.id)
                {
                    nodes.Add(syllabus);
                    await FindByPid(syllabus, nodes, nodedata);
                }
            }
            return nodes;
        }


        private async Task<List<Syllabus>> SaveOrUpdateToNodes(List<SyllabusNode> syllabusNodes)
        {
            List<Syllabus> syllabuses;

            List<Syllabus> data = await _azureCosmos.FindByDict<Syllabus>(new Dictionary<string, object> { { "id", syllabusNodes[0].code } });
            if (data.IsEmpty())
            {
                var syllabus = new Syllabus { id = syllabusNodes[0].code, code = syllabusNodes[0].code, children = new List<SyllabusNode>() };
                await _azureCosmos.SaveOrUpdate(syllabus);
                data = new List<Syllabus>() { syllabus };
            }
            List<SyllabusNode> syllabusNodes1 = new List<SyllabusNode>();
            if (data.IsNotEmpty())
            {
                syllabusNodes1.AddRange(data[0].children);
                //replace
                for (int i = 0; i < data[0].children.Count; i++)
                {
                    for (int j = 0; j < syllabusNodes.Count; j++)
                    {
                        if (data[0].children[i].id == syllabusNodes[j].id && data[0].children[i].code == syllabusNodes[j].code)
                        {
                            syllabusNodes1.Remove(data[0].children[i]);
                            syllabusNodes1.Add(syllabusNodes[j]);
                            syllabusNodes.Remove(syllabusNodes[j]);
                        }
                    }
                }
                data[0].children = syllabusNodes1;

                //新增
                data[0].children.AddRange(syllabusNodes);

                if (data[0].children.IsNotEmpty())
                {
                    var len = data[0].children.Count;
                    for (int i = 0; i < len; i++)
                    {
                        if (data[0].children[i].items.IsNotEmpty())
                        {
                            data[0].children[i].item = true;
                        }
                        if (data[0].children[i].resources.IsNotEmpty())
                        {
                            data[0].children[i].resource = true;
                        }
                    }
                }
                syllabuses = await _azureCosmos.SaveOrUpdateAll<Syllabus>(data);

            }
            else
            {
                throw new BizException("保存失败", ResponseCode.FAILED);

            }

            return syllabuses;
        }

        private async Task<List<SyllabusTree>> SaveOrUpdateToTree(List<SyllabusTree> request)
        {
            List<SyllabusNode> nodes = new List<SyllabusNode>();
            Syllabus syllabus = new Syllabus();

            TreeToList(request, nodes);
            // List<SyllabusNode> nods = nodes.ToJson().FromJson<List<SyllabusNode>>() ;
            syllabus.children = nodes;
            syllabus.id = request[0].code;
            syllabus.code = request[0].code;
            await _azureCosmos.SaveOrUpdate<Syllabus>(syllabus);
            List<SyllabusTree> treess = ListToTree(nodes);
            return treess;
        }

        private List<SyllabusNode> TreeToList(List<SyllabusTree> trees, List<SyllabusNode> nodes)
        {
            int index = 0;
            foreach (SyllabusTree tr in trees)
            {
                tr.order = index;
                index++;
            }
            trees = trees.OrderBy(x => x.order).ToList();
            nodes.AddRange(trees.ToJsonString().ToObject<List<SyllabusNode>>());

            foreach (SyllabusTree tree in trees)
            {
                if (null != tree.children && tree.children.Count > 0)
                {
                    TreeToList(tree.children, nodes);
                }
            }
            return nodes;
        }
        private List<SyllabusTree> ListToTree(List<SyllabusNode> noes)
        {
            List<SyllabusTree> list = noes.ToJsonString().ToObject<List<SyllabusTree>>();
            //var lookup = list.ToDictionary(n => n.RowKey, n => n);

            var res = from r in list group r by r.id into g select g;
            Dictionary<string, SyllabusTree> blockDict = new Dictionary<string, SyllabusTree>();
            foreach (var s in res)
            {
                blockDict.TryAdd(s.First().id, s.First());
            }
            return GetChild(list, blockDict);
        }

        private List<SyllabusTree> GetChild(List<SyllabusTree> list, Dictionary<string, SyllabusTree> dict)
        {
            //  list = list.OrderBy(m => m.Order).ToList();
            List<SyllabusTree> trees = new List<SyllabusTree>();
            trees = trees.OrderBy(x => x.order).ToList();
            foreach (SyllabusTree node in list)
            {
                bool flag = dict.TryGetValue(node.pid, out SyllabusTree syllabus);
                if (flag && syllabus != null)
                {
                    syllabus.children.Add(node);
                }
                else
                {
                    trees.Add(node);
                }
            }
            return trees;
        }

        private async Task<List<Syllabus>> DeleteSyllabus(Dictionary<string, object> request)
        {
            Dictionary<string, object> dict = new Dictionary<string, object>();
            foreach (KeyValuePair<string, object> keyValuePair in request)
            {
                dict.Add("children[*]." + keyValuePair.Key, keyValuePair.Value);
            }

            List<Syllabus> data = await _azureCosmos.FindByDict<Syllabus>(dict);
            Dictionary<string, Syllabus> syllabuses = new Dictionary<string, Syllabus>();
            List<Syllabus> data1 = new List<Syllabus>();
            if (data.Count > 0)
            {
                data.ForEach(x => syllabuses.TryAdd(x.id, x));
                data = new List<Syllabus>(syllabuses.Values);

                //是否全删
                bool flg = false;
                foreach (Syllabus item in data)
                {
                    List<SyllabusNode> nodedata = item.children;
                    if (request.TryGetValue("id", out object id))
                    {
                        foreach (SyllabusNode syllabus in nodedata)
                        {
                            if (syllabus.id == id.ToString())
                            {
                                List<SyllabusNode> nodes = new List<SyllabusNode>();
                                nodes.Add(syllabus);
                                await FindByPid(syllabus, nodes, nodedata);
                                nodedata = nodes;
                                break;
                            }
                        }
                    }
                    List<SyllabusNode> syllabusNodes = new List<SyllabusNode>();
                    syllabusNodes.AddRange(item.children);
                    for (int i = 0; i < item.children.Count; i++)
                    {
                        for (int j = 0; j < nodedata.Count; j++)
                        {
                            if (item.children[i].id == nodedata[j].id)
                            {
                                syllabusNodes.Remove(item.children[i]);
                            }
                        }
                    }
                    item.children = syllabusNodes;
                    if (item.children.Count == 0)
                    {
                        await _azureCosmos.DeleteAsync<Syllabus>(item.id, item.code);
                        flg = true;
                    }
                }
                if (!flg)
                {
                    data1 = await _azureCosmos.SaveOrUpdateAll<Syllabus>(data);

                }


                //if (flg)
                //{

                //    return builder.Data("全部删除成功").build();
                //}
                //else
                //{
                //    List<Syllabuses> data1 = await azureCosmosDBRepository.SaveOrUpdateAll<Syllabuses>(data);
                //    return builder.Data(data1).build();
                //}


            }
            return data1;
        }
        #endregion


    }
}
