using Azure.Cosmos;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Mvc;
using System;
using System.Collections.Generic;
using System.IdentityModel.Tokens.Jwt;
using System.Text.Json;
using System.Threading.Tasks;
using TEAMModelOS.Models.Dto;
using TEAMModelOS.Models.TeacherInfo;
using TEAMModelOS.SDK;
using TEAMModelOS.SDK.DI;
using TEAMModelOS.SDK.Extension;
using TEAMModelOS.SDK.Helper.Common.CollectionHelper;
using TEAMModelOS.SDK.Helper.Common.StringHelper;

namespace TEAMModelOS.Controllers
{
    [ProducesResponseType(StatusCodes.Status200OK)]
    [ProducesResponseType(StatusCodes.Status400BadRequest)]
    //[Authorize(Roles = "IES5")]
    [Route("teacher/comment")]
    [ApiController]
    public class CommentController :ControllerBase
    {
        private readonly AzureCosmosFactory _azureCosmos;
        public CommentController(AzureCosmosFactory azureCosmos)
        {
            _azureCosmos = azureCosmos;
        }

        /// <summary>
        /// 添加教师评语快捷回复
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        [ProducesDefaultResponseType]
        [HttpPost("add-comment")]
        public async Task<IActionResult> AddComment(CommentsDto request)
        {
            var id = "Comment-" + request.TEAMModelId.Replace("#", "");
            ResponseBuilder builder = ResponseBuilder.custom();
            List<Comment> comments = await _azureCosmos.FindByDict<Comment>(new Dictionary<string, object> { { "code", request.TEAMModelId }, { "id", id } });
            Comment comment = new Comment();
            if (comments.IsEmpty())
            {
                comment.id = id;
                comment.code = request.TEAMModelId;
                comment.comment.Add(request.comment);
                comments.Add(comment);
            }
            else {
                comment = comments[0];
                comment.comment.Add(request.comment);
            }
            //builder.Data(await _azureCosmos.SaveOrUpdate(comment));
            //return builder.build();
            return Ok(await _azureCosmos.SaveOrUpdate(comment));
        }

        /// <summary>
        /// 查询教师评语罐头
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        [ProducesDefaultResponseType]
        [HttpPost("find-comment")]
        public async Task<IActionResult> findComment(JsonElement requert)
        {
            var client = _azureCosmos.GetCosmosClient();
            if (!requert.TryGetProperty("school_code", out JsonElement school_code)) return BadRequest();
            //var (id, name, picture, _) = HttpContext.GetAuthTokenInfo();
            if (!requert.TryGetProperty("id_token", out JsonElement id_token)) return BadRequest();
            var jwt = new JwtSecurityToken(id_token.GetString());
            if (!jwt.Payload.Iss.Equals("account.teammodel", StringComparison.Ordinal)) return BadRequest();
            var id = jwt.Payload.Sub;

            List<object> comments = new List<object>();
            var query = $"select c.id,c.comment from c";
            await foreach (var item in client.GetContainer("TEAMModelOS", "Teacher").GetItemQueryStreamIterator(queryText: query, requestOptions: new QueryRequestOptions() { PartitionKey = new PartitionKey($"Comment-{id}") }))
            {
                using var json = await JsonDocument.ParseAsync(item.ContentStream);

                if (json.RootElement.TryGetProperty("_count", out JsonElement count) && count.GetUInt16() > 0)
                {
                    foreach (var obj in json.RootElement.GetProperty("Documents").EnumerateArray())
                    {
                        comments.Add(obj.ToObject<object>());
                    }
                }
            }

            return Ok(new { comments});
            // request.TryAdd("PartitionKey", request.lang);
            /*ResponseBuilder builder = ResponseBuilder.custom();

            List<Comment> data = new List<Comment>();
            if (StringHelper.getKeyCount(request) > 0)
            {
                data = await _azureCosmos.FindByDict<Comment>(request);
            }
            else
            {
                return builder.Error(ResponseCode.PARAMS_ERROR, "参数异常！").build();
            }
            return builder.Data(data).build();*/
        }
        /// <summary>
        /// 更新保存教师评语罐头,如果评语列表为空则删除
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        [ProducesDefaultResponseType]
        [HttpPost("upsert-comment")]
        public async Task<IActionResult> UpsertComment(Comment request)
        {
            // request.TryAdd("PartitionKey", request.lang);
            ResponseBuilder builder = ResponseBuilder.custom();
            Comment comment = null;
            if (request.comment.Count > 0)
            {
                if (string.IsNullOrEmpty(request.id)) {
                    request.id  = "Comment-" + request.code.Replace("#", "");
                }
                comment = await _azureCosmos.SaveOrUpdate<Comment>(request);
            }
            else {
                if (!string.IsNullOrEmpty(request.id))
                {
                    IdPk idPk = await _azureCosmos.DeleteAsync<Comment>(request.id, request.code);
                }
                   
            }
            //return builder.Data(comment).build();
            return Ok(comment);
        }

         
    }
}
