using Microsoft.AspNetCore.Mvc;
using System;
using System.Collections.Generic;
using System.Text;
using System.Threading.Tasks;
using TEAMModelOS.SDK;
using TEAMModelOS.SDK.DI;
using TEAMModelOS.SDK.Helper.Common.CollectionHelper;
using TEAMModelOS.Models;
using System.Linq;
using TEAMModelOS.SDK.Helper.Common.JsonHelper;
using TEAMModelOS.SDK.Helper.Common.ReflectorExtensions;
using System.Text.Json;
using Newtonsoft.Json.Linq;
using Newtonsoft.Json;

using System.Collections;
using TEAMModelOS.SDK.Context.Exception;
using System.Reflection;
using Microsoft.Azure.Cosmos.Linq;
using TEAMModelOS.Models.SchoolInfo;
using TEAMModelOS.Models.TeacherInfo;

namespace TEAMModelOS.Controllers
{
    [Route("api/[controller]")]
    [ApiController]
    public class SchoolUserController : ControllerBase
    {
        private AzureCosmosFactory _azureCosmos;
        private AzureStorageFactory _azureStorage;
        public SchoolUserController(AzureCosmosFactory azureCosmos, AzureStorageFactory azureStorage)
        {
            _azureCosmos = azureCosmos;
            _azureStorage = azureStorage;
        }

        /// <summary>
        /// 取得學校使用者(複數)
        /// </summary>
        [HttpPost("GetSchoolUser")]
        public async Task<BaseResponse> GetSchoolUser(JsonElement request)
        {
            ResponseBuilder builder = ResponseBuilder.custom();
            //request.@params.TryGetValue("code", out object partitionKey);
            List<SchoolUser> su = await _azureCosmos.FindByDict<SchoolUser>(request);
            return builder.Data(su).build();
        }

        /// <summary>
        /// 以使用者ID取得學校使用者資料
        /// </summary>
        [HttpPost("GetSingleSchoolUser")]
        public async Task<BaseResponse> GetSingleSchoolUser(JsonElement request)
        {
            ResponseBuilder builder = ResponseBuilder.custom();
            //request.TryGetProperty("schoolCode", out JsonElement schoolCodeObj);
            //request.TryGetProperty("id", out JsonElement idObj);
            //request.TryGetProperty("joinStatus", out JsonElement joinStatusObj);
            string schoolCode = request.TryGetProperty("schoolCode", out JsonElement schoolCodeObj) ? schoolCodeObj.ToString() : null;
            string id = request.TryGetProperty("id", out JsonElement idObj) ? idObj.ToString() : null;
            string joinStatusStr = request.TryGetProperty("joinStatus", out JsonElement joinStatusObj) ? joinStatusObj.ToString() : null;
            int joinStatus = (joinStatusStr != null) ? Convert.ToInt32(joinStatusStr) : -1;
            Dictionary<string, object> userDict = new Dictionary<string, object>
            {
                { "id", id}
            };
            if (!string.IsNullOrEmpty(schoolCode))
            {
                string partitionKey = schoolCode + "-" + "SchoolUser";
                userDict.Add("schoolCode", schoolCode);
                userDict.Add("code", partitionKey);
            }
            if (joinStatus >= 0)
            {
                userDict.Add("joinStatus", joinStatus);
            }
            List<SchoolUser> su = await _azureCosmos.FindByDict<SchoolUser>(userDict);

            return builder.Data(su).build();
        }

        /// <summary>
        /// 設定權限
        /// </summary>
        /// <returns>使用者資訊(更新後)</returns>
        [HttpPost("SetAuthToSchoolUser")]
        public async Task<BaseResponse> SetAuthToSchoolUser(JsonElement request)
        {
            ResponseBuilder builder = ResponseBuilder.custom();

            request.TryGetProperty("schoolCode", out JsonElement schoolCode);
            request.TryGetProperty("ids", out JsonElement idsJson);
            List<string> ids = new List<string>();
            var idsArr = JArray.Parse(idsJson.ToString());
            foreach (var idInfo in idsArr)
            {
                ids.Add(idInfo["id"].ToString());
            }
            request.TryGetProperty("authList", out JsonElement authListObj);
            List<string> authList = JsonConvert.DeserializeObject<List<string>>(authListObj.ToString());

            //取得已加入該校的使用者 (joinStatus=1: [加入狀態]  1:已加入)
            Dictionary<string, object> userDict = new Dictionary<string, object>
            {
                { "schoolCode",  schoolCode.ToString()},
                { "id",  ids},
                { "joinStatus",  1}
            };
            List<SchoolUser> suListResponse = new List<SchoolUser>(); //返回值
            List<SchoolUser> suList = await _azureCosmos.FindByDict<SchoolUser>(userDict);
            //設定權限
            if (suList.Count > 0)
            {
                foreach (SchoolUser su in suList)
                {
                    su.authority = authList;
                    suListResponse.Add(su);
                    await _azureCosmos.Update(su);
                }
            }
            return builder.Data(suListResponse).build();
        }

        /// <summary>
        /// 更新使用者資料
        /// </summary>
        /// <returns>使用者資訊(更新後)</returns>
        [HttpPost("updParamToSchoolUser")]
        public async Task<BaseResponse> updParamToSchoolUser(JsonElement request)
        {
            ResponseBuilder builder = ResponseBuilder.custom();
            request.TryGetProperty("schoolCode", out JsonElement schoolCode);
            request.TryGetProperty("id", out JsonElement id);
            request.TryGetProperty("param", out JsonElement paramJson);
            //dynamic param = JsonConvert.DeserializeObject<dynamic>(paramJson.ToString());
            dynamic param = JObject.Parse(paramJson.ToString()) as dynamic;


            //取得該校的該使用者
            Dictionary<string, object> userDict = new Dictionary<string, object>
            {
                { "schoolCode",  schoolCode.ToString()},
                { "id",  id.ToString()}
            };
            SchoolUser suResponse = new SchoolUser(); //返回值
            List<SchoolUser> suList = await _azureCosmos.FindByDict<SchoolUser>(userDict);
            //更新資料
            if (suList.Count > 0)
            {
                foreach (SchoolUser su in suList)
                {
                    PropertyInfo[] properties = typeof(SchoolUser).GetProperties();
                    foreach (PropertyInfo property in properties)
                    {
                        IDictionary<string, JToken> dict = param;
                        if (dict.ContainsKey(property.Name))
                        {
                            string updVal = param[property.Name].ToString();
                            if (property.Name == "remindDate" || property.Name == "joinDate" || property.Name == "lastLoginDate" || property.Name == "joinStatus")
                            {
                                su.GetType().GetProperty(property.Name).SetValue(su, Int32.Parse(updVal));
                            }
                            else
                            {
                                su.GetType().GetProperty(property.Name).SetValue(su, updVal);
                            }
                        }
                    }
                    await _azureCosmos.Update(su);
                    suResponse = su;
                }
            }
            return builder.Data(suResponse).build();
        }

        /// <summary>
        /// 移除學校使用者
        /// </summary>
        [HttpPost("rmvSchoolUser")]
        public async Task<BaseResponse> RmvSchoolUser(JsonElement request)
        {
            ResponseBuilder builder = ResponseBuilder.custom();
            request.TryGetProperty("schoolCode", out JsonElement schoolCode);
            request.TryGetProperty("Ids", out JsonElement Ids);
            //以TEAMModelId(array)取得該校的使用者
            Dictionary<string, object> userDict = new Dictionary<string, object>
            {
                { "schoolCode",  schoolCode.ToString()},
                { "id",  Ids}
            };
            List<SchoolUser> suList = await _azureCosmos.FindByDict<SchoolUser>(userDict);
            //移除
            if (suList.Count > 0)
            {
                foreach (SchoolUser su in suList)
                {
                    await _azureCosmos.DeleteAsync(su);
                }
            }
            return builder.build();
        }

        /// <summary>
        /// 取得權限總列表
        /// </summary>
        [HttpPost("GetSchoolSettingAuthorityList")]
        public async Task<BaseResponse> GetSchoolSettingAuthorityList()
        {
            ResponseBuilder builder = ResponseBuilder.custom();
            Dictionary<string, object> dict = new Dictionary<string, object>
            {
                { "PartitionKey",  "authority"}
            };
            List<Authority> schoolAuthorityList = await _azureStorage.FindListByDict<Authority>(dict);
            return builder.Data(schoolAuthorityList).build();
        }

        /// <summary>
        /// 追加學校使用者(複數)、無返回值
        /// </summary>
        [HttpPost("AddSchoolUser")]
        public async Task<BaseResponse> AddSchoolUser(JsonElement request)
        {
            ResponseBuilder builder = ResponseBuilder.custom();
 /*           Dictionary<string, object> dict = new Dictionary<string, object>();
            var emobj = request.EnumerateObject();
            while (emobj.MoveNext())
            {
                dict[emobj.Current.Name] = emobj.Current.Value;
            }*/
            try
            {
               
                request.TryGetProperty("schoolCode", out JsonElement schoolCode);
                request.TryGetProperty("userlist", out JsonElement userList);
                List<SchoolUser> addUserlist = JsonConvert.DeserializeObject<List<SchoolUser>>(userList.ToString());
                //整形
                foreach (SchoolUser addUser in addUserlist)
                {
                    addUser.pk = "SchoolUser";
                    addUser.schoolCode = schoolCode.ToString();
                    addUser.jobTitle = "teacher"; //先寫死...
                    addUser.role = new List<string>();
                    addUser.role.Add("teacher"); //先寫死...
                    addUser.crtDate = addUser.remindDate = new DateTimeOffset(DateTime.Now).ToUnixTimeSeconds();
                    //addUser.crtDate = addUser.remindDate = (int)DateTime.Now.Subtract(new DateTime(1970, 1, 1, 0, 0, 0, DateTimeKind.Local)).TotalSeconds;
                    addUser.authority = new List<string>();
                    addUser.code = schoolCode + "-SchoolUser";
                }
                await _azureCosmos.SaveOrUpdateAll<SchoolUser>(addUserlist);
                return builder.build();
            }
            catch (Exception e)
            {
                return builder.Error(ResponseCode.FAILED, e.StackTrace).build();
            }
        }

        /// <summary>
        /// 追加學校使用者(單數)、返回該使用者資料
        /// </summary>
        [HttpPost("AddSchoolUserSingle")]
        public async Task<BaseResponse> AddSchoolUserSingle(JsonElement request)
        {
            ResponseBuilder builder = ResponseBuilder.custom();

            try
            {

                request.TryGetProperty("schoolCode", out JsonElement schoolCode);
                request.TryGetProperty("userinfo", out JsonElement userinfo);
                SchoolUser addUserinfo = JsonConvert.DeserializeObject<SchoolUser>(userinfo.ToString());
                //整形
                addUserinfo.pk = "SchoolUser";
                addUserinfo.schoolCode = schoolCode.ToString();
                addUserinfo.crtDate = new DateTimeOffset(DateTime.Now).ToUnixTimeSeconds();
                addUserinfo.authority = new List<string>();
                addUserinfo.code = schoolCode + "-SchoolUser";

                await _azureCosmos.SaveOrUpdate<SchoolUser>(addUserinfo);

                return builder.Data(addUserinfo).build();
            }
            catch (Exception e)
            {
                return builder.Error(ResponseCode.FAILED, e.StackTrace).build();
            }
        }
    }
}
