using Newtonsoft.Json;
using ProtoBuf;
using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.Text;
using System.Text.Json.Serialization;
using TEAMModelOS.SDK.Context.Attributes.Azure;
using TEAMModelOS.SDK.DI;

namespace TEAMModelOS.Models.CommonInfo
{
    /// <summary>
    ///Teaching materials 教学材料 --- 知识点，知识块
    /// </summary>
    [CosmosDB(Database = "TEAMModelOS", Name = "Common", Cache = true )]
    [ProtoContract]
    public class Knowledge: ID
    {
        public Knowledge()
        {
            points = new List<string>();
        }
        /// <summary>
        /// 学校编码 如果是教师个人知识点为醍摩豆id 否则为学校编码 
        /// </summary>
        [PartitionKey]
        [Required(ErrorMessage = "{0} 必须填写")]
        public string code { get; set; }
        // [PartitionKey(name = "Knowledge")]
        public string pk { get; set; }
        public int? ttl { get; set; }
        [ProtoMember(1)]
        public string id { get; set; }
        /// <summary>
        /// 标记为知识块，0=知识块 ，1 知识点
        /// </summary>
        [ProtoMember(2)]
        [Required(ErrorMessage = "{0} 必须填写")]
        public int type { get; set; } = 0;
        /// <summary>
        /// 名称
        /// </summary>
        [ProtoMember(3)]
        [Required(ErrorMessage = "{0} 必须填写")]
        public string name { get; set; }
        /// <summary>
        /// 描述
        /// </summary>
        [ProtoMember(4)]
        [Required(ErrorMessage = "{0} 必须填写")]
        public string alias { get; set; }

        /// <summary>
        /// 学科
        /// </summary>
        ///[Required(ErrorMessage = "{0} 必须填写")]
        [ProtoMember(5)]
        public string subjectId { get; set; }

       


        [ProtoMember(7)]
        [Required(ErrorMessage = "{0} 必须填写")]
        public int order { get; set; }

        /// <summary>
        /// 删除状态0 删除 1 有效
        /// </summary>
        [ProtoMember(8)]
        [Required(ErrorMessage = "{0} 必须填写")]
        public int status { get; set; } = 1;

        /// <summary>
        /// 知识点 id
        /// </summary>
        [ProtoMember(9)]
        [Required(ErrorMessage = "{0} 必须填写")]
        public List<string> points { get; set; }


        /// <summary>
        ///  来自于标准表 Rowkey
        /// </summary>
        [ProtoMember(10)]
        [Required(ErrorMessage = "{0} 必须填写")]
        public string knowledgeId { get; set; }

        /// <summary>
        /// 0 标准，1自建，2.....
        /// </summary>
        [ProtoMember(11)]
        [Required(ErrorMessage = "{0} 必须填写")]
        public int source { get; set; }

        /// <summary>
        /// 学段 私有的没有学段
        /// </summary>
        ///[Required(ErrorMessage = "{0} 必须填写")]
        [ProtoMember(12)]
        public string periodId { get; set; }
        public int ownCount { get; set; }
        public int openCount { get; set; }
    }
}
