using ProtoBuf;
using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.Text;
using TEAMModelOS.Models.CommonInfo;
using TEAMModelOS.SDK.Context.Attributes.Azure;
using TEAMModelOS.SDK.DI;

namespace TEAMModelOS.Models.StudentInfo
{

    /// <summary>
    /// 作业数据
    /// </summary>
    [ProtoContract]
    [CosmosDB(Database = "TEAMModelOS", Name = "Student")]
    public class HomeworkRecord : ID
    {
        /// <summary>
        /// 学生id
        /// </summary>
        [PartitionKey]
        [Required(ErrorMessage = "{0} 必须填写")]
        public string code { get; set; }
        public string pk { get; set; }
        public int? ttl { get; set; }
        public HomeworkRecord() {
            content = new List<HomeWorkContent>();
            stuCmt = new List<StudentComment>();
            classroom = new ClassroomItem();
           // nativeroom = new ClassroomItem();
            tchCmt = new HomeWorkComment();
        }
        /// <summary>
        /// 作业活动id
        /// </summary>
        [ProtoMember(1)]
        public string id { get; set; }

        /// <summary>
        /// 上课班级
        /// </summary>
        [ProtoMember(5)]
        public ClassroomItem classroom { get; set; }

        ///// <summary>
        ///// 原生班级
        ///// </summary>
        //[ProtoMember(6)]
        //public ClassroomItem nativeroom{ get; set; }

        /// <summary>
        /// 提交时间
        /// </summary>
        [ProtoMember(9)]
        public long submitTime { get; set; }

        /// <summary>
        /// 分数
        /// </summary>
        [ProtoMember(10)]
        public double score { get; set; }

        /// <summary>
        /// 作业内容
        /// </summary>
        [ProtoMember(11)]
        public List<HomeWorkContent> content { get; set; }


        /// <summary>
        /// 作业评论 （同学互评）
        /// </summary>
        [ProtoMember(12)]
        public List<StudentComment> stuCmt { get; set; }


        /// <summary>
        /// 教师评语
        /// </summary>
        [ProtoMember(13)]
        public HomeWorkComment tchCmt { get; set; }
    }
    [ProtoContract]
    public class ClassroomItem {
        [ProtoMember(1)]
        public string name { get; set; }
        [ProtoMember(2)]
        public string code { get; set; }

    }

    /// <summary>
    /// 作业内容
    /// </summary>
    [ProtoContract]
    public class HomeWorkContent {

        /// <summary>
        /// 作答记录
        /// </summary>
        [ProtoMember(1)]
        public string answers { get; set; }

        /// <summary>
        ///  其他模式url 及name
        /// </summary>
        [ProtoMember(2)]
        public List<ProcessRes> resource { get; set; }

    }


    [ProtoContract]
    public class StudentComment
    {
        public StudentComment()
        {
            reply = new List<Reply>();
        }

        /// <summary>
        /// 评论id
        /// </summary>
        [ProtoMember(1)]
        public string commentid { get; set; }

        /// <summary>
        /// 评论者id 醍摩豆或studenId
        /// </summary>
        [ProtoMember(2)]
        public string fromId { get; set; }

        /// <summary>
        /// 被评论者id 醍摩豆或studenId
        /// </summary>
        [ProtoMember(3)]
        public string toId { get; set; }

        /// <summary>
        /// 学生打分
        /// </summary>
        [ProtoMember(4)]
        public double? score { get; set; }

        /// <summary>
        /// 评论者身份 教师 学生
        /// </summary>
        [ProtoMember(5)]
        public string identity { get; set; }

        /// <summary>
        /// 评语
        /// </summary>
        [ProtoMember(6)]
        public string comment { get; set; }

        /// <summary>
        /// 评论时间
        /// </summary>
        [ProtoMember(7)]
        public long createTime { get; set; }

        ///// <summary>
        ///// 评论回复
        ///// </summary>
        [ProtoMember(8)]
        public List<Reply>  reply { get; set; }

    }



    /// <summary>
    /// 作业评论
    /// </summary>
    [ProtoContract]
    public class HomeWorkComment {
       

        /// <summary>
        /// 评论者身份 教师 学生
        /// </summary>
        [ProtoMember(1)]
        public string identity { get; set; }

        /// <summary>
        /// 评语
        /// </summary>
        [ProtoMember(2)]
        public string comment { get; set; }

        /// <summary>
        /// 评论时间
        /// </summary>
        [ProtoMember(3)]
        public long createTime { get; set; }


        /// <summary>
        /// 醍摩豆id 或者学生id
        /// </summary>
        [ProtoMember(4)]
        public string TEAMModelId { get; set; }

        ///// <summary>
        ///// 评论回复
        ///// </summary>
        //public List<Reply>  reply { get; set; }

    }

    /// <summary>
    /// 评论回复
    /// </summary>
    [ProtoContract]
    public class Reply
    {
        /// <summary>
        /// 评论者id 醍摩豆或studenId
        /// </summary>
        [ProtoMember(1)]
        public string fromId { get; set; }

        /// <summary>
        /// 被评论者id 醍摩豆或studenId
        /// </summary>
        [ProtoMember(2)]
        public string toId { get; set; }

        /// <summary>
        /// 评论者身份 教师 学生
        /// </summary>
        [ProtoMember(3)]
        public string identity { get; set; }

        /// <summary>
        /// 评语
        /// </summary>
        [ProtoMember(4)]
        public string comment { get; set; }

        [ProtoMember(5)]
        public long createTime { get; set; }

    }
}
