using HtmlAgilityPack;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using TEAMModelOS.Models.CommonInfo;
using TEAMModelOS.Models.Dto;
using TEAMModelOS.SDK.Context.Exception;
using TEAMModelOS.SDK.Helper.Common.CollectionHelper;
using TEAMModelOS.SDK.Helper.Common.StringHelper;
using TEAMModelOS.SDK.Helper.Security.ShaHash;


namespace TEAMModelOS.Services
{
    public class HtmlAnalyzeService
    {
        public LangConfig langConfig { get; set; }
        public HtmlAnalyzeService(LangConfig _langConfig) {
            langConfig = _langConfig;
        }


        /// <summary>
        /// 处理 【答案】【解析】【结束】以及题型标签中包含的空格字符
        /// </summary>
        /// <param name="html"></param>
        /// <returns></returns>
        public string BlankProcess(string html) {

            string ans = langConfig.Item.Start + langConfig.Item.AnswerTag + langConfig.Item.End;
            string als = langConfig.Item.Start + langConfig.Item.AnalysisTag + langConfig.Item.End;
            string end = langConfig.Item.Start + langConfig.Item.EndedTag + langConfig.Item.End;
            string[] ansarry = ans.Select(s => s.ToString()).ToArray();
            string[] alsarry = als.Select(s => s.ToString()).ToArray();
            string[] endarry = end.Select(s => s.ToString()).ToArray();
            
            string ansReg = string.Join("\\s*",ansarry);
            string alsReg = string.Join("\\s*", alsarry);
            string endReg = string.Join("\\s*", endarry);
            html = Regex.Replace(html, ansReg, ans);
            html = Regex.Replace(html, alsReg, als);
            html = Regex.Replace(html, endReg, end);
            string blankReg = "\\s*";
            foreach (string value in langConfig.Item.Type.Values) {
                string tag = langConfig.Item.Start+"\\s*" + "\\d+\\s*"+ string.Join("\\s*", value.Select(s=>s.ToString()).ToArray())+"\\s*"+langConfig.Item.End;
                var m = Regex.Match(html, tag);
                while (m.Success)
                {
                    string blankStr = Regex.Replace(m.Value, blankReg, "");
                    html = html.Replace(m.Value, blankStr);
                    m = m.NextMatch();
                }
            }
            return html;
        }

        public  List<ItemInfo> AnalyzeWordAsync(string html)
        {
            string mathjax = "<script type=\"text/javascript\" src=\"http://cdn.mathjax.org/mathjax/latest/MathJax.js?config=TeX-AMS-MML_HTMLorMML\"></script>";
            html = html.Replace(mathjax, "");
            //去除class 以及span标签"
            string classpattern = "class=\"([^\"]*)\"";
            html = Regex.Replace(html, classpattern, "");
            string pattern = "<span([^>]{0,})>";
            html = Regex.Replace(html, pattern, "");
            html = html.Replace(" close=\"\" separators=\" | \">", "");
            html = html.Replace("\t", " ").Replace("<span>", "").Replace("</span>", "").Replace("dir=\"ltr\"", "");
            //处理 【答案】【解析】【结束】标签中包含的空格字符
            html = BlankProcess(html);

            Dictionary<string, List<string>> TestInType = ConvertTest(html);
            List<ItemInfo> tests = new List<ItemInfo>();
            foreach (string key in TestInType.Keys)
            {
                switch (key)
                {
                    case "single":
                        List<ItemInfo> exercisesSingle = SingleConvert(key, TestInType[key]);
                        exercisesSingle.ForEach(x => { x.pShaCode = x.shaCode; });
                        tests.AddRange(exercisesSingle); break;
                    case "multiple":
                        List<ItemInfo> exercisesMultiple = MultipleConvert(key, TestInType[key]);
                        exercisesMultiple.ForEach(x => { x.pShaCode = x.shaCode; });
                        tests.AddRange(exercisesMultiple); break;
                    case "judge":
                        List<ItemInfo> exercisesJudge = JudgeConvert(key, TestInType[key]);
                        exercisesJudge.ForEach(x => { x.pShaCode = x.shaCode; });
                        tests.AddRange(exercisesJudge); break;
                    case "complete":
                        List<ItemInfo> exercisesComplete = CompleteConvert(key, TestInType[key]);
                        exercisesComplete.ForEach(x => { x.pShaCode = x.shaCode; });
                        tests.AddRange(exercisesComplete); break;
                    case "subjective":
                        List<ItemInfo> exercisesSubjective = SubjectiveConvert(key, TestInType[key]);
                        exercisesSubjective.ForEach(x => { x.pShaCode = x.shaCode; });
                        tests.AddRange(exercisesSubjective); break;
                    case "compose":
                        List<ItemInfo> exercisesCompose = ComposeConvert(key, TestInType[key]);
                        exercisesCompose.ForEach(x => { x.pShaCode = x.shaCode; });
                        tests.AddRange(exercisesCompose);
                        break;
                    default: break;
                }
            }

            return tests.OrderBy(x => x.order).ToList();
        }

        private  List<ItemInfo> SingleConvert(string TypeKey, List<string> list)
        {
            List<ItemInfo> testInfos = OptionProcess(TypeKey, list);
            return testInfos;
        }

        private  List<ItemInfo> MultipleConvert(string TypeKey, List<string> list)
        {
            List<ItemInfo> testInfos = OptionProcess(TypeKey, list);
            return testInfos;
        }

        private  List<ItemInfo> JudgeConvert(string TypeKey, List<string> list)
        {
            List<ItemInfo> testInfos = OptionProcess(TypeKey, list);
            return testInfos;
        }


        private  List<ItemInfo> CompleteConvert(string TypeKey, List<string> list)
        {
            List<ItemInfo> testInfos = CompleteProcess(TypeKey, list);
            return testInfos;
        }
        private  List<ItemInfo> CompleteProcess(string TypeKey, List<string> tests)
        {
            //List<string> tests = ConvertTest(testHtml);
            List<ItemInfo> testInfos = ConvertTestInfo(tests, TypeKey);
            HtmlDocument doc = new HtmlDocument();
            foreach (ItemInfo testInfo in testInfos)
            {
                List<string> ans = new List<string>();
                testInfo.question = testInfo.question.Replace(langConfig.Item.Start + langConfig.Item. AnalysisTag + langConfig.Item.End, "").Replace(langConfig.Item.Start + langConfig.Item.AnswerTag + langConfig.Item.End, "");
                string regRex = langConfig.Item.Start + "([\\s\\S]*?)" + langConfig.Item.End;
                List<ReplaceDto> replaces = new List<ReplaceDto>();
                var m = Regex.Match(testInfo.question, regRex);
                int index = 1;
                while (m.Success)
                {
                    string an = m.Groups[1].ToString();
                    doc.LoadHtml(an);
                    string anstr = doc.DocumentNode.InnerText;
                    string nbsp = "";
                    int length = System.Text.Encoding.Default.GetBytes(anstr).Length;
                    for (int i = 0; i < length * 3; i++)
                    {
                        nbsp += "&nbsp;";
                    }
                    ReplaceDto replaceDto = new ReplaceDto { oldstr = langConfig.Item.Start + an + langConfig.Item.End, newstr = "<underline style='word-break: break-word' data=\"" + index + "\"><u>" + nbsp + "</u></underline>" };
                    replaces.Add(replaceDto);
                    ans.Add(an);
                    m = m.NextMatch();
                    index++;
                }
                string textImg = testInfo.question;
                //消除答案
                foreach (ReplaceDto replace in replaces)
                {
                    testInfo.question = testInfo.question.Replace(replace.oldstr, replace.newstr);
                    testInfo.question = HtmlHelper.DoUselessTag(testInfo.question);
                    //只要题干文字和图片 
                    //不加underline标记
                    textImg = testInfo.question.Replace(replace.oldstr, "");

                }
                textImg = HtmlHelper.DoTextImg(textImg);
                testInfo.shaCode = ShaHashHelper.GetSHA1(textImg);
                //处理解析
                testInfo.explain = testInfo.explain.Replace(langConfig.Item.Start + langConfig.Item.AnalysisTag + langConfig.Item.End, "");//.Replace(Start + EndedTag + End, "");
                testInfo.explain = HtmlHelper.DoUselessTag(testInfo.explain);
                testInfo.answer.AddRange(ans);
            }
            return testInfos;
        }

        private  List<ItemInfo> OptionProcess(string typeKey, List<string> list)
        {
            string[] optionsKeys = langConfig.Item.Options.Select(s => s.ToString()).ToArray();
            List<ItemInfo> testInfos = ConvertTestInfo(list, typeKey);
            foreach (ItemInfo testInfo in testInfos)
            {
                string optsRgex = "";
                if (testInfo.question.Contains(langConfig.Item.Start + langConfig.Item.AnswerTag + langConfig.Item.End))
                {
                    optsRgex = optionsKeys[0] + "\\s*(\\．|\\.|\\、|\\:|\\：)([\\s\\S]*?)" + langConfig.Item.Start + langConfig.Item.AnswerTag + langConfig.Item.End;
                }
                else if (!testInfo.question.Contains(langConfig.Item.Start + langConfig.Item.AnswerTag + langConfig.Item.End) && testInfo.question.Contains(langConfig.Item.Start + langConfig.Item.AnalysisTag + langConfig.Item.End))
                {
                    optsRgex = optionsKeys[0] + "\\s*(\\．|\\.|\\、|\\:|\\：)([\\s\\S]*?)" + langConfig.Item.Start + langConfig.Item.AnalysisTag + langConfig.Item.End;
                }
                else {
                    optsRgex = optionsKeys[0] + "\\s*(\\．|\\.|\\、|\\:|\\：)([\\s\\S]*?).*" ;
                }
                
                
                string optsHtml = Regex.Match(testInfo.question, optsRgex).Value;
                //HtmlDocument doc = new HtmlDocument(); +".*";
                //doc.LoadHtml(optsHtml);
                //optsHtml = doc.DocumentNode.InnerText;
                //处理选项
                StringBuilder textImg = new StringBuilder();
                for (int i = 0; i < optionsKeys.Length - 1; i++)
                {
                    string optRgex = optionsKeys[i] + "\\s*(\\．|\\.|\\、|\\:|\\：)([\\s\\S]*?)" + optionsKeys[i + 1] + "\\s*(\\．|\\.|\\、|\\:|\\：)";
                    string optHtml = Regex.Match(optsHtml, optRgex).Value;
                    //if (string.IsNullOrEmpty(optHtml)) {
                    //    optRgex = optionsKeys[i]+"(\\．|\\.|\\、|\\:|\\：).*";
                    //    optHtml = Regex.Match(optsHtml, optRgex).Value;
                    //}
                    if (!string.IsNullOrEmpty(optHtml))
                    {
                        optHtml = optHtml.Substring(2, optHtml.Length - 4);
                        optHtml = HtmlHelper.DoUselessTag(optHtml);
                        textImg.Append(HtmlHelper.DoTextImg(optHtml));
                        testInfo.option.Add(new CodeValue { code = optionsKeys[i], value = optHtml });
                        //testInfo.option.Add(new Dictionary<string, string> { { "code", optionsKeys[i] },{ "value", optHtml } });
                        //testInfo.option.TryAdd(optionsKeys[i], optHtml);
                    }
                    else
                    {
                        if (optsHtml.Contains(langConfig.Item.Start + langConfig.Item.AnswerTag + langConfig.Item.End))
                        {
                            optRgex = optionsKeys[i] + "\\s*(\\．|\\.|\\、|\\:|\\：)([\\s\\S]*?)" + langConfig.Item.Start + langConfig.Item.AnswerTag + langConfig.Item.End;
                        }
                        else if (!optsHtml.Contains(langConfig.Item.Start + langConfig.Item.AnswerTag + langConfig.Item.End) && optsHtml.Contains(langConfig.Item.Start + langConfig.Item.AnalysisTag + langConfig.Item.End))
                        {
                            optRgex = optionsKeys[i] + "\\s*(\\．|\\.|\\、|\\:|\\：)([\\s\\S]*?)" + langConfig.Item.Start + langConfig.Item.AnalysisTag + langConfig.Item.End;
                        }
                        else
                        {
                            optRgex = optionsKeys[i] + "\\s*(\\．|\\.|\\、|\\:|\\：).*";
                        }
                        optHtml = Regex.Match(optsHtml, optRgex).Value;
                        if (!string.IsNullOrEmpty(optHtml))
                        {
                            // optHtml = optHtml.Substring(2, optHtml.Length - 6);
                            optHtml = optHtml.Substring(2 ).Replace(langConfig.Item.Start + langConfig.Item.AnswerTag + langConfig.Item.End, "").Replace(langConfig.Item.Start + langConfig.Item.AnalysisTag + langConfig.Item.End, "");
                            optHtml = HtmlHelper.DoUselessTag(optHtml);
                            textImg.Append(HtmlHelper.DoTextImg(optHtml));
                            testInfo.option.Add(new CodeValue { code = optionsKeys[i], value = optHtml });
                            //testInfo.option.Add(new Dictionary<string, string> { { "code", optionsKeys[i] }, { "value", optHtml } });
                            //testInfo.option.TryAdd(optionsKeys[i], optHtml);
                        }
                    }
                }
                //处理题干
                try {
                    testInfo.question = testInfo.question.Replace(optsHtml, "").Replace(langConfig.Item.Start + langConfig.Item.AnswerTag + langConfig.Item.End, "");
                } catch (Exception e) { throw new BizException("第"+testInfo.order+"题，选项设置错误", ResponseCode.PARAMS_ERROR,e.StackTrace); }
                testInfo.question = HtmlHelper.DoUselessTag(testInfo.question);
                textImg.Append(HtmlHelper.DoTextImg(testInfo.question));
                testInfo.shaCode = ShaHashHelper.GetSHA1(textImg.ToString());
                List<string> answers = testInfo.answer;
                HashSet<string> ans = new HashSet<string>();
                //处理答案
                for (int i = 0; i < answers.Count; i++)
                {
                    string Answer = answers[i].Replace(langConfig.Item.Start + langConfig.Item.AnswerTag + langConfig.Item.End, "").Replace(langConfig.Item.Start + langConfig.Item.AnalysisTag + langConfig.Item.End, "").TrimStart().TrimEnd();
                    Answer.Select(s => s.ToString()).ToList().ForEach(x =>
                    {
                        ans.Add(x);
                    });
                }
                testInfo.answer = ans.ToList();
                //处理解析
                testInfo.explain = testInfo.explain.Replace(langConfig.Item.Start + langConfig.Item.AnalysisTag + langConfig.Item.End, "");//.Replace(Start + EndedTag + End, "");
                testInfo.explain = HtmlHelper.DoUselessTag(testInfo.explain);
            }
            return testInfos;
        }


        private  List<ItemInfo> SubjectiveConvert(string TypeKey, List<string> tests)
        {
            // List<string> tests = ConvertTest(testHtml);
            List<ItemInfo> testInfos = ConvertTestInfo(tests, TypeKey);

            foreach (ItemInfo testInfo in testInfos)
            {
                testInfo.question = testInfo.question.Replace(langConfig.Item.Start + langConfig.Item.AnalysisTag + langConfig.Item.End, "").Replace(langConfig.Item.Start + langConfig.Item.AnswerTag + langConfig.Item.End, "");
                testInfo.question = HtmlHelper.DoUselessTag(testInfo.question);
                StringBuilder textImg = new StringBuilder(HtmlHelper.DoTextImg(testInfo.question));
                testInfo.shaCode = ShaHashHelper.GetSHA1(textImg.ToString());
                for (int i = 0; i < testInfo.answer.Count; i++)
                {
                    testInfo.answer[i] = testInfo.answer[i].Replace(langConfig.Item.Start + langConfig.Item.AnswerTag + langConfig.Item.End, "").Replace(langConfig.Item.Start + langConfig.Item.AnalysisTag + langConfig.Item.End, "");
                    testInfo.answer[i] = HtmlHelper.DoUselessTag(testInfo.answer[i]);
                }
                testInfo.explain = testInfo.explain.Replace(langConfig.Item.Start + langConfig.Item.AnalysisTag + langConfig.Item.End, "");//.Replace(Start + EndedTag + End, "");
                testInfo.explain = HtmlHelper.DoUselessTag(testInfo.explain);
            }
            return testInfos;
        }

        private  List<ItemInfo> ComposeConvert(string TypeKey, List<string> list)
        {
            List<ItemInfo> exerciseDtos = new List<ItemInfo>();
            foreach (string html in list)
            {
                ItemInfo exercise = new ItemInfo() { type = TypeKey };
               
                List<string> test = new List<string>();
                foreach (string k in langConfig.Item.Type.Keys)
                {
                    string Rex = langConfig.Item.Start + "\\d+[^" + langConfig.Item.Start + langConfig.Item.End + "]*" + langConfig.Item.Type[TypeKey] + langConfig.Item.End + "[\\s\\S]+?(?=\\s*" + langConfig.Item.Start + "\\d+[^" + langConfig.Item.Start + langConfig.Item.End + "]*" + langConfig.Item.Type[k] + langConfig.Item.End + "|$)";
                    var mm = Regex.Match(html, Rex);
                    while (mm.Success)
                    {
                        test.Add(mm.Value);
                        mm = mm.NextMatch();
                    }
                }
                string ls = test.OrderBy(x => x.Length).First();

                //处理题号
                string digtRex = langConfig.Item.Start + "\\d+" + langConfig.Item.Type[TypeKey] + langConfig.Item.End;
                var m = Regex.Match(ls, digtRex);
                //int index = 1;

                while (m.Success)
                {
                    string ord = m.Value.Replace(langConfig.Item.Start, "").Replace(langConfig.Item.End, "").Replace(langConfig.Item.Type[TypeKey], "");
                    int.TryParse(ord, out int order);
                    exercise.order = order;
                    m = m.NextMatch();
                }

                string testinfo = html.Replace( ls, "").Replace(langConfig.Item.Start + langConfig.Item.EndedTag + langConfig.Item.End, "");
                ls =     Regex.Replace(ls, langConfig.Item.Start + "\\d+" + langConfig.Item.Type[TypeKey] + langConfig.Item.End, "");
                exercise.question = HtmlHelper.DoUselessTag(ls/*.Replace(ComposeStart, "").Replace(ComposeEnd, "")*/);
                //.Replace(ComposeTag, Start);
                //获取综合题的材料加每个小题的sha1Code
                string testQs = HtmlHelper.DoTextImg(exercise.question);
                List<ItemInfo> dtos = AnalyzeWordAsync(testinfo);
                if (dtos.IsNotEmpty())
                {
                    dtos.ForEach(x => { testQs = testQs + x.shaCode; });
                    exercise.shaCode = ShaHashHelper.GetSHA1(testQs);
                    dtos.ForEach(x => { x.pShaCode = exercise.shaCode; x.lite = true; });
                    exercise.children.AddRange(dtos);
                }
                if (exercise.children.IsNotEmpty()) {
                    exercise.children = exercise.children.OrderBy(x => x.order).ToList();
                }
                exerciseDtos.Add(exercise);
            }
            return exerciseDtos;
        }
        /// <summary>
        /// 处理单选，多选，判断
        /// </summary>
        /// <param name="tests"></param>
        /// <param name="TypeKey"></param>
        /// <returns></returns>
        public  List<ItemInfo> ConvertTestInfo(List<string> tests, string TypeKey)
        {
            List<ItemInfo> testInfos = new List<ItemInfo>();
            foreach (string html in tests)
            {
                Dictionary<string, string> regex = new Dictionary<string, string>();
                Dictionary<string, string> question = new Dictionary<string, string> { { "Summary", langConfig.Item.Start+ "\\d+" + langConfig.Item.Type[TypeKey] + langConfig.Item.End + "|" + langConfig.Item.Start + langConfig.Item.AnswerTag + langConfig.Item.End }, { "Answer", langConfig.Item.Start + langConfig.Item.AnswerTag + langConfig.Item.End + "|" + langConfig.Item.Start + langConfig.Item.AnalysisTag + langConfig.Item.End }, { "Analysis", langConfig.Item.Start + langConfig.Item.AnalysisTag + langConfig.Item.End/* + "|" + Start + EndedTag + End */} };
                Dictionary<string, string> compquestion = new Dictionary<string, string> { { "Summary", langConfig.Item.Start + "\\d+" + langConfig.Item.Type[TypeKey] + langConfig.Item.End + "|" + langConfig.Item.Start + langConfig.Item.AnalysisTag + langConfig.Item.End }, { "Analysis", langConfig.Item.Start + langConfig.Item.AnalysisTag + langConfig.Item.End /* + "|" + Start + EndedTag + End */ } };
                ItemInfo test = new ItemInfo();
                test.type = TypeKey;
                List<string> keys = new List<string>();
                if (TypeKey.Equals("complete"))
                {
                    keys = compquestion.Keys.ToList();
                    regex = compquestion;
                }
                else
                {
                    keys = question.Keys.ToList();
                    regex = question;
                }
                foreach (string key in keys)
                {
                    string[] tags = regex[key].Split("|");
                    string RegexStr ="";
                    if (key == "Summary")
                    {
                        if (html.Contains(langConfig.Item.Start + langConfig.Item.AnswerTag + langConfig.Item.End))
                        {
                            RegexStr = tags[0] + "([\\s\\S]*?)" + tags[1];
                        }
                        else if (!html.Contains(langConfig.Item.Start + langConfig.Item.AnswerTag + langConfig.Item.End) && html.Contains(langConfig.Item.Start + langConfig.Item.AnalysisTag + langConfig.Item.End))
                        {
                            RegexStr = tags[0] + "([\\s\\S]*?)" + langConfig.Item.Start + langConfig.Item.AnalysisTag + langConfig.Item.End;
                        }
                        else {
                            RegexStr = tags[0] +".*";
                            //RegexStr = tags[0] + "([\\s\\S]*?)";
                        }
                    }
                    else if (key == "Answer")
                    {
                        if (html.Contains(langConfig.Item.Start + langConfig.Item.AnalysisTag + langConfig.Item.End))
                        {
                            RegexStr = tags[0] + "([\\s\\S]*?)" + tags[1];
                        }
                        else {
                            RegexStr = tags[0] + ".*";
                            //RegexStr = tags[0] + "([\\s\\S]*?)";
                        }
                    }
                    else if (key == "Analysis") {
                        RegexStr = tags[0] + ".*";
                        //RegexStr = tags[0] + "([\\s\\S]*?)";
                    }
                    Match mt = Regex.Match(html, RegexStr);
                    switch (key)
                    {
                        case "Summary":
                            //处理题号
                            string digtRex = langConfig.Item.Start + "\\d+" + langConfig.Item.Type[TypeKey] + langConfig.Item.End;
                            var m = Regex.Match(mt.Value, digtRex);
                            //int index = 1;
                           
                            while (m.Success)
                            {
                                string ord = m.Value.Replace(langConfig.Item.Start, "").Replace(langConfig.Item.End, "").Replace(langConfig.Item.Type[TypeKey], "");
                                int.TryParse(ord,out int  order);
                                test.order = order;
                                m = m.NextMatch();
                            }
                            test.question=Regex.Replace(mt.Value, langConfig.Item.Start + "\\d+" + langConfig.Item.Type[TypeKey] + langConfig.Item.End, "");
                            
                            //test.question = mt.Value.Replace(Start + "\\d+" + langConfig.Item.Type[TypeKey] + End, "");
                            break;
                        case "Answer":
                            string Answer = mt.Value;
                            //单选或多选,判断答案 脱html标签
                            if (TypeKey.Equals("single") || TypeKey.Equals("multiple") || TypeKey.Equals("judge"))
                            {
                                HtmlDocument doc = new HtmlDocument();
                                doc.LoadHtml(mt.Value);
                                Answer = doc.DocumentNode.InnerText;
                            }
                            test.answer = new List<string>() { Answer }; break;
                        case "Analysis":
                            test.explain = mt.Value; break;
                        default: break;
                    }
                }
                testInfos.Add(test);
            }
            return testInfos;
        }

        //private  Dictionary<string, string> Test1Type = new Dictionary<string, string> {
        //    { "single", "【单选题】|【结束】" }, { "multiple", "【多选题】|【结束】" },
        //    { "judge", "【判断题】|【结束】" }, { "complete", "【填空题】|【结束】" },
        //    { "subjective", "【问答题】|【结束】" } , { "compose", "【综合题】|【完结】" }};
        //private  Dictionary<string, string> langConfig.Item.Type = new Dictionary<string, string> {
        //    { "compose", "综合题" },
        //    { "single", "单选题" }, { "multiple", "多选题" },
        //    { "judge", "判断题" }, { "complete", "填空题" },
        //    { "subjective", "问答题" } ,};
        //private  Dictionary<string, string> TestENType = new Dictionary<string, string> {
        //    { "compose", "compose" },
        //    { "single", "single" }, { "multiple", "multiple" },
        //    { "judge", "judge" }, { "complete", "complete" },
        //    { "subjective", "subjective" } };
        //private  string Start = "【";
        //private  string End = "】";
        //private  string AnswerTag = "答案";
        //private  string AnalysisTag = "解析";
        //private  string EndedTag = "结束";
        //private  string Options = "ABCDEFGHIJKLMN";

        /// <summary>
        /// 解析题型
        /// </summary>
        /// <param name="testHtml"></param>
        /// <returns></returns>
        public  Dictionary<string, List<string>> ConvertTest(string testHtml)
        {

            Dictionary<string, List<string>> TestInType = new Dictionary<string, List<string>>();
            foreach (string key in langConfig.Item.Type.Keys)
            {
                List<string> tests = new List<string>();
                string regRex = "";
                if (key == "compose")
                {
                    regRex = langConfig.Item.Start + "\\d+" + langConfig.Item.Type[key] + langConfig.Item.End + "([\\s\\S]*?)" + langConfig.Item.Start + langConfig.Item.EndedTag + langConfig.Item.End;
                    var m = Regex.Match(testHtml, regRex);
                    while (m.Success)
                    {
                        string testInfo =m.Value;
                        testHtml= testHtml.Replace(testInfo, "");
                        tests.Add(testInfo);
                        m = m.NextMatch();
                    }
                }
                else {
                    ///【\d+[^【】]*题】[\s\S]+?(?=\s*【\d+[^【】]*题】|$)
                    regRex = langConfig.Item.Start + "\\d+[^" + langConfig.Item.Start + langConfig.Item.End + "]*" + langConfig.Item.Type[key] + langConfig.Item.End + "[\\s\\S]+?(?=\\s*" + langConfig.Item.Start + "\\d+[^" + langConfig.Item.Start + langConfig.Item.End + "]*" + langConfig.Item.Type[key] + langConfig.Item.End + "|$)";
                    //  string[] tags = TestType[key].Split("|");
                    //  string regRex = tags[0] + "([\\s\\S]*?)" + tags[1];
                    var m = Regex.Match(testHtml, regRex);
                    //int index = 1;
                    while (m.Success)
                    {
                        tests.Add(m.Value);
                        m = m.NextMatch();
                    }
                    //处理结尾匹配问题
                    if (tests.IsNotEmpty())
                    {
                        // string last = tests.Last();
                        //再次匹配，拿到长度最短的。
                        for (int i = 0; i < tests.Count; i++)
                        {
                            List<string> test = new List<string>();
                            foreach (string k in langConfig.Item.Type.Keys)
                            {
                                string Rex = langConfig.Item.Start + "\\d+[^" + langConfig.Item.Start + langConfig.Item.End + "]*" + langConfig.Item.Type[key] + langConfig.Item.End + "[\\s\\S]+?(?=\\s*" + langConfig.Item.Start + "\\d+[^" + langConfig.Item.Start + langConfig.Item.End + "]*" + langConfig.Item.Type[k] + langConfig.Item.End + "|$)";
                                var mm = Regex.Match(tests[i], Rex);
                                while (mm.Success)
                                {
                                    test.Add(mm.Value);
                                    mm = mm.NextMatch();
                                }
                            }
                            string ls = test.OrderBy(x => x.Length).First();
                            tests[i] = ls;
                        }
                    }
                }
                TestInType.Add(key, tests);
            }
            return TestInType;
        }
    }
    class ReplaceDto
    {
        public string oldstr { get; set; }
        public string newstr { get; set; }
    }
}
